@@ -6,17 +6,17 @@
 # Copyright (C) 2009 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -26,11 +26,17 @@ import zlib
 
 import gst
 
-from morituri.common import common, task, gstreamer
+from morituri.common import common
+from morituri.common import gstreamer as cgstreamer
+from morituri.common import log
+from morituri.common import task
+
+from morituri.extern.task import gstreamer
 
 # checksums are not CRC's. a CRC is a specific type of checksum.
 
-class ChecksumTask(gstreamer.GstPipelineTask):
+
+class ChecksumTask(log.Loggable, gstreamer.GstPipelineTask):
     """
     I am a task that calculates a checksum of the decoded audio data.
 
@@ -42,28 +48,40 @@ class ChecksumTask(gstreamer.GstPipelineTask):
     # this object needs a main loop to stop
     description = 'Calculating checksum'
 
-    def __init__(self, path, frameStart=0, frameLength=-1):
+    def __init__(self, path, sampleStart=0, sampleLength=-1):
         """
-        A frame is considered a set of samples for each channel;
-        ie 16 bit stereo is 4 bytes per frame.
-        If frameLength < 0 it is treated as 'unknown' and calculated.
+        A sample is considered a set of samples for each channel;
+        ie 16 bit stereo is 4 bytes per sample.
+        If sampleLength < 0 it is treated as 'unknown' and calculated.
 
         @type  path:       unicode
-        @type  frameStart: int
-        @param frameStart: the frame to start at
+        @type  sampleStart: int
+        @param sampleStart: the sample to start at
         """
+
+        # sampleLength can be e.g. -588 when it is -1 * SAMPLES_PER_FRAME
+
         assert type(path) is unicode, "%r is not unicode" % path
 
+        self.logName = "ChecksumTask 0x%x" % id(self)
+
         # use repr/%r because path can be unicode
-        self.debug('Creating checksum task on %r from %d to %d',
-            path, frameStart, frameLength)
+        if sampleLength < 0:
+            self.debug(
+                'Creating checksum task on %r from sample %d until the end',
+                path, sampleStart)
+        else:
+            self.debug(
+                'Creating checksum task on %r from sample %d for %d samples',
+                path, sampleStart, sampleLength)
+
         if not os.path.exists(path):
-            raise IndexError, '%r does not exist' % path
+            raise IndexError('%r does not exist' % path)
 
         self._path = path
-        self._frameStart = frameStart
-        self._frameLength = frameLength
-        self._frameEnd = None
+        self._sampleStart = sampleStart
+        self._sampleLength = sampleLength
+        self._sampleEnd = None
         self._checksum = 0
         self._bytes = 0 # number of bytes received
         self._first = None
@@ -72,7 +90,7 @@ class ChecksumTask(gstreamer.GstPipelineTask):
 
         self.checksum = None # result
 
-        gstreamer.removeAudioParsers()
+        cgstreamer.removeAudioParsers()
 
     ### gstreamer.GstPipelineTask implementations
 
@@ -81,12 +99,12 @@ class ChecksumTask(gstreamer.GstPipelineTask):
             filesrc location="%s" !
             decodebin name=decode ! audio/x-raw-int !
             appsink name=sink sync=False emit-signals=True
-            ''' % common.quoteParse(self._path).encode('utf-8')
+            ''' % gstreamer.quoteParse(self._path).encode('utf-8')
 
     def paused(self):
         sink = self.pipeline.get_by_name('sink')
 
-        if self._frameLength < 0:
+        if self._sampleLength < 0:
             self.debug('query duration')
             try:
                 length, qformat = sink.query_duration(gst.FORMAT_DEFAULT)
@@ -98,12 +116,15 @@ class ChecksumTask(gstreamer.GstPipelineTask):
             if qformat == gst.FORMAT_BYTES:
                 self.debug('query returned in BYTES format')
                 length /= 4
-            self.debug('total length: %r', length)
-            self._frameLength = length - self._frameStart
-            self.debug('audio frame length is %r', self._frameLength)
+            self.debug('total sample length of file: %r', length)
+            self._sampleLength = length - self._sampleStart
+            self.debug('sampleLength is queried as %d samples',
+                self._sampleLength)
         else:
-            self.debug('frameLength known, is %d' % self._frameLength)
-        self._frameEnd = self._frameStart + self._frameLength - 1
+            self.debug('sampleLength is known, and is %d samples' %
+                self._sampleLength)
+        self._sampleEnd = self._sampleStart + self._sampleLength - 1
+        self.debug('sampleEnd is sample %d' % self._sampleEnd)
 
         self.debug('event')
 
@@ -111,16 +132,18 @@ class ChecksumTask(gstreamer.GstPipelineTask):
         # the segment end only is respected since -good 0.10.14.1
         event = gst.event_new_seek(1.0, gst.FORMAT_DEFAULT,
             gst.SEEK_FLAG_FLUSH,
-            gst.SEEK_TYPE_SET, self._frameStart,
-            gst.SEEK_TYPE_SET, self._frameEnd + 1) # half-inclusive interval
-        gst.debug('CRCing %r from sector %d to sector %d' % (
+            gst.SEEK_TYPE_SET, self._sampleStart,
+            gst.SEEK_TYPE_SET, self._sampleEnd + 1) # half-inclusive interval
+        self.debug('CRCing %r from sector %d to sector %d' % (
             self._path,
-            self._frameStart / common.SAMPLES_PER_FRAME,
-            (self._frameEnd + 1) / common.SAMPLES_PER_FRAME))
-        # FIXME: sending it with frameEnd set screws up the seek, we don't get
+            self._sampleStart / common.SAMPLES_PER_FRAME,
+            (self._sampleEnd + 1) / common.SAMPLES_PER_FRAME))
+        # FIXME: sending it with sampleEnd set screws up the seek, we don't get
         # everything for flac; fixed in recent -good
         result = sink.send_event(event)
         self.debug('event sent, result %r', result)
+        if not result:
+            self.error('Failed to select samples with GStreamer seek event')
         sink.connect('new-buffer', self._new_buffer_cb)
         sink.connect('eos', self._eos_cb)
 
@@ -139,25 +162,34 @@ class ChecksumTask(gstreamer.GstPipelineTask):
         self.debug('scheduled setting to play')
 
     def stopped(self):
+        self.debug('stopped')
         if not self._last:
             # see http://bugzilla.gnome.org/show_bug.cgi?id=578612
+            self.debug(
+                'not a single buffer gotten, setting exception EmptyError')
             self.setException(common.EmptyError('not a single buffer gotten'))
+            return
         else:
             self._checksum = self._checksum % 2 ** 32
-            self.debug("last offset %r", self._last.offset)
+            self.debug("last buffer's sample offset %r", self._last.offset)
+            self.debug("last buffer's sample size %r", len(self._last) / 4)
             last = self._last.offset + len(self._last) / 4 - 1
-            self.debug("last sample: %r", last)
-            self.debug("frame end: %r", self._frameEnd)
-            self.debug("frame length: %r", self._frameLength)
+            self.debug("last sample offset in buffer: %r", last)
+            self.debug("requested sample end: %r", self._sampleEnd)
+            self.debug("requested sample length: %r", self._sampleLength)
             self.debug("checksum: %08X", self._checksum)
             self.debug("bytes: %d", self._bytes)
-            if self._frameEnd != last:
-                print 'ERROR: did not get all frames, %d missing' % (
-                    self._frameEnd - last)
+            if self._sampleEnd != last:
+                msg = 'did not get all samples, %d of %d missing' % (
+                    self._sampleEnd - last, self._sampleEnd)
+                self.warning(msg)
+                self.setException(common.MissingFrames(msg))
+                return
 
         self.checksum = self._checksum
 
     ### subclass methods
+
     def do_checksum_buffer(self, buf, checksum):
         """
         Subclasses should implement this.
@@ -165,17 +197,18 @@ class ChecksumTask(gstreamer.GstPipelineTask):
         raise NotImplementedError
 
     ### private methods
+
     def _new_buffer_cb(self, sink):
         buf = sink.emit('pull-buffer')
         gst.log('received new buffer at offset %r with length %r' % (
             buf.offset, buf.size))
         if self._first is None:
             self._first = buf.offset
-            self.debug('first sample is %r', self._first)
+            self.debug('first sample is sample offset %r', self._first)
         self._last = buf
 
         assert len(buf) % 4 == 0, "buffer is not a multiple of 4 bytes"
-        
+
         # FIXME: gst-python 0.10.14.1 doesn't have adapter_peek/_take wrapped
         # see http://bugzilla.gnome.org/show_bug.cgi?id=576505
         self._adapter.push(buf)
@@ -188,9 +221,9 @@ class ChecksumTask(gstreamer.GstPipelineTask):
             self._bytes += len(buf)
 
             # update progress
-            frame = self._first + self._bytes / 4
-            framesDone = frame - self._frameStart
-            progress = float(framesDone) / float((self._frameLength))
+            sample = self._first + self._bytes / 4
+            samplesDone = sample - self._sampleStart
+            progress = float(samplesDone) / float((self._sampleLength))
             # marshall to the main thread
             self.schedule(0, self.setProgress, progress)
 
@@ -200,6 +233,7 @@ class ChecksumTask(gstreamer.GstPipelineTask):
         self.debug('eos, scheduling stop')
         self.schedule(0, self.stop)
 
+
 class CRC32Task(ChecksumTask):
     """
     I do a simple CRC32 check.
@@ -210,6 +244,7 @@ class CRC32Task(ChecksumTask):
     def do_checksum_buffer(self, buf, checksum):
         return zlib.crc32(buf, checksum)
 
+
 class AccurateRipChecksumTask(ChecksumTask):
     """
     I implement the AccurateRip checksum.
@@ -219,8 +254,9 @@ class AccurateRipChecksumTask(ChecksumTask):
 
     description = 'Calculating AccurateRip checksum'
 
-    def __init__(self, path, trackNumber, trackCount, frameStart=0, frameLength=-1):
-        ChecksumTask.__init__(self, path, frameStart, frameLength)
+    def __init__(self, path, trackNumber, trackCount, sampleStart=0,
+            sampleLength=-1):
+        ChecksumTask.__init__(self, path, sampleStart, sampleLength)
         self._trackNumber = trackNumber
         self._trackCount = trackCount
         self._discFrameCounter = 0 # 1-based
@@ -244,10 +280,10 @@ class AccurateRipChecksumTask(ChecksumTask):
                 checksum += common.SAMPLES_PER_FRAME * 5 * values[0]
                 checksum &= 0xFFFFFFFF
                 return checksum
- 
+
         # on last track, skip last 5 CD frames
         if self._trackNumber == self._trackCount:
-            discFrameLength = self._frameLength / common.SAMPLES_PER_FRAME
+            discFrameLength = self._sampleLength / common.SAMPLES_PER_FRAME
             if self._discFrameCounter > discFrameLength - 5:
                 self.debug('skipping frame %d', self._discFrameCounter)
                 return checksum
@@ -259,12 +295,13 @@ class AccurateRipChecksumTask(ChecksumTask):
             checksum &= 0xFFFFFFFF
             # offset = self._bytes / 4 + i + 1
             # if offset % common.SAMPLES_PER_FRAME == 0:
-            #    print 'THOMAS: frame %d, ends before %d, last value %08x, CRC %08x' % (
-            #        offset / common.SAMPLES_PER_FRAME, offset, value, sum)
+            #   print 'frame %d, ends before %d, last value %08x, CRC %08x' % (
+            #     offset / common.SAMPLES_PER_FRAME, offset, value, sum)
 
         return checksum
 
-class TRMTask(gstreamer.GstPipelineTask):
+
+class TRMTask(task.GstPipelineTask):
     """
     I calculate a MusicBrainz TRM fingerprint.
 
@@ -276,7 +313,7 @@ class TRMTask(gstreamer.GstPipelineTask):
 
     def __init__(self, path):
         if not os.path.exists(path):
-            raise IndexError, '%s does not exist' % path
+            raise IndexError('%s does not exist' % path)
 
         self.path = path
         self._trm = None
@@ -289,7 +326,6 @@ class TRMTask(gstreamer.GstPipelineTask):
             trm name=trm !
             appsink name=sink sync=False emit-signals=True''' % self.path
 
-
     def parsed(self):
         sink = self.pipeline.get_by_name('sink')
         sink.connect('new-buffer', self._new_buffer_cb)
@@ -308,11 +344,11 @@ class TRMTask(gstreamer.GstPipelineTask):
 
     # FIXME: can't move this to base class because it triggers too soon
     # in the case of checksum
+
     def bus_eos_cb(self, bus, message):
         gst.debug('eos, scheduling stop')
         self.schedule(0, self.stop)
 
-
     def bus_tag_cb(self, bus, message):
         taglist = message.parse_tag()
         if 'musicbrainz-trmid' in taglist.keys():
