@@ -6,17 +6,17 @@
 # Copyright (C) 2009 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -25,7 +25,12 @@ import os
 import shutil
 import tempfile
 
-from morituri.common import common, task, log, gstreamer
+from morituri.common import common, log
+from morituri.common import gstreamer as cgstreamer
+from morituri.common import task as ctask
+
+from morituri.extern.task import task, gstreamer
+
 
 class Profile(object):
     name = None
@@ -40,6 +45,7 @@ class Profile(object):
         """
         pass
 
+
 class FlacProfile(Profile):
     name = 'flac'
     extension = 'flac'
@@ -47,6 +53,7 @@ class FlacProfile(Profile):
     lossless = True
 
     # FIXME: we should do something better than just printing ERRORS
+
     def test(self):
 
         # here to avoid import gst eating our options
@@ -67,6 +74,8 @@ class FlacProfile(Profile):
         return True
 
 # FIXME: ffenc_alac does not have merge_tags
+
+
 class AlacProfile(Profile):
     name = 'alac'
     extension = 'alac'
@@ -74,28 +83,35 @@ class AlacProfile(Profile):
     lossless = True
 
 # FIXME: wavenc does not have merge_tags
+
+
 class WavProfile(Profile):
     name = 'wav'
     extension = 'wav'
     pipeline = 'wavenc'
     lossless = True
 
+
 class WavpackProfile(Profile):
     name = 'wavpack'
     extension = 'wv'
     pipeline = 'wavpackenc bitrate=0 name=tagger'
     lossless = True
 
+
 class MP3Profile(Profile):
     name = 'mp3'
     extension = 'mp3'
     pipeline = 'lame name=tagger quality=0 ! id3v2mux'
     lossless = False
 
+
 class MP3VBRProfile(Profile):
     name = 'mp3vbr'
     extension = 'mp3'
-    pipeline = 'lame name=tagger quality=0 vbr=new vbr-mean-bitrate=192 ! id3v2mux'
+    pipeline = 'lame name=tagger ' \
+               'vbr-quality=0 vbr=new vbr-mean-bitrate=192 ! ' \
+               'id3v2mux'
     lossless = False
 
 
@@ -107,22 +123,23 @@ class VorbisProfile(Profile):
 
 
 PROFILES = {
-    'wav':     WavProfile,
-    'flac':    FlacProfile,
-    'alac':    AlacProfile,
+    'wav': WavProfile,
+    'flac': FlacProfile,
+    'alac': AlacProfile,
     'wavpack': WavpackProfile,
 }
 
 LOSSY_PROFILES = {
-    'mp3':     MP3Profile,
-    'mp3vbr':  MP3VBRProfile,
-    'vorbis':  VorbisProfile,
+    'mp3': MP3Profile,
+    'mp3vbr': MP3VBRProfile,
+    'vorbis': VorbisProfile,
 }
 
 ALL_PROFILES = PROFILES.copy()
 ALL_PROFILES.update(LOSSY_PROFILES)
 
-class EncodeTask(gstreamer.GstPipelineTask):
+
+class EncodeTask(ctask.GstPipelineTask):
     """
     I am a task that encodes a .wav file.
     I set tags too.
@@ -146,7 +163,7 @@ class EncodeTask(gstreamer.GstPipelineTask):
         assert type(inpath) is unicode, "inpath %r is not unicode" % inpath
         assert type(outpath) is unicode, \
             "outpath %r is not unicode" % outpath
-        
+
         self._inpath = inpath
         self._outpath = outpath
         self._taglist = taglist
@@ -158,6 +175,8 @@ class EncodeTask(gstreamer.GstPipelineTask):
         self.description = "Encoding %s" % what
         self._profile.test()
 
+        cgstreamer.removeAudioParsers()
+
     def getPipelineDesc(self):
         return '''
             filesrc location="%s" !
@@ -166,9 +185,9 @@ class EncodeTask(gstreamer.GstPipelineTask):
             level name=level !
             %s ! identity name=identity !
             filesink location="%s" name=sink''' % (
-                common.quoteParse(self._inpath).encode('utf-8'),
+                gstreamer.quoteParse(self._inpath).encode('utf-8'),
                 self._profile.pipeline,
-                common.quoteParse(self._outpath).encode('utf-8'))
+                gstreamer.quoteParse(self._outpath).encode('utf-8'))
 
     def parsed(self):
         tagger = self.pipeline.get_by_name('tagger')
@@ -216,6 +235,14 @@ class EncodeTask(gstreamer.GstPipelineTask):
 
         bus.connect('message::element', self._message_element_cb)
         self._level = self.pipeline.get_by_name('level')
+
+        # set an interval that is smaller than the duration
+        # FIXME: check level and make sure it emits level up to the last
+        # sample, even if input is small
+        interval = 1000000000L
+        if interval < duration:
+            interval = duration / 2
+        self._level.set_property('interval', interval)
         # add a probe so we can track progress
         # we connect to level because this gives us offset in samples
         srcpad = self._level.get_static_pad('src')
@@ -266,8 +293,11 @@ class EncodeTask(gstreamer.GstPipelineTask):
             self.peak = math.sqrt(math.pow(10, self._peakdB / 10.0))
         else:
             self.warning('No peak found, something went wrong!')
+            # workaround for when the file is too short to have volume ?
+            # self.peak = 0.0
 
-class TagReadTask(gstreamer.GstPipelineTask):
+
+class TagReadTask(ctask.GstPipelineTask):
     """
     I am a task that reads tags.
 
@@ -285,7 +315,7 @@ class TagReadTask(gstreamer.GstPipelineTask):
         """
         """
         assert type(path) is unicode, "path %r is not unicode" % path
-        
+
         self._path = path
 
     def getPipelineDesc(self):
@@ -293,7 +323,7 @@ class TagReadTask(gstreamer.GstPipelineTask):
             filesrc location="%s" !
             decodebin name=decoder !
             fakesink''' % (
-                common.quoteParse(self._path).encode('utf-8'))
+                gstreamer.quoteParse(self._path).encode('utf-8'))
 
     def bus_eos_cb(self, bus, message):
         self.debug('eos, scheduling stop')
@@ -301,9 +331,15 @@ class TagReadTask(gstreamer.GstPipelineTask):
 
     def bus_tag_cb(self, bus, message):
         taglist = message.parse_tag()
-        self.taglist = taglist
+        self.debug('tag_cb, %d tags' % len(taglist.keys()))
+        if not self.taglist:
+            self.taglist = taglist
+        else:
+            import gst
+            self.taglist = self.taglist.merge(taglist, gst.TAG_MERGE_REPLACE)
 
-class TagWriteTask(task.Task):
+
+class TagWriteTask(ctask.LoggableTask):
     """
     I am a task that retags an encoded file.
     """
@@ -317,7 +353,7 @@ class TagWriteTask(task.Task):
         """
         assert type(inpath) is unicode, "inpath %r is not unicode" % inpath
         assert type(outpath) is unicode, "outpath %r is not unicode" % outpath
-        
+
         self._inpath = inpath
         self._outpath = outpath
         self._taglist = taglist
@@ -328,12 +364,14 @@ class TagWriteTask(task.Task):
         # here to avoid import gst eating our options
         import gst
 
+        # FIXME: this hardcodes flac; we should be using the correct
+        #        tag element instead
         self._pipeline = gst.parse_launch('''
             filesrc location="%s" !
             flactag name=tagger !
             filesink location="%s"''' % (
-                common.quoteParse(self._inpath).encode('utf-8'),
-                common.quoteParse(self._outpath).encode('utf-8')))
+                gstreamer.quoteParse(self._inpath).encode('utf-8'),
+                gstreamer.quoteParse(self._outpath).encode('utf-8')))
 
         # set tags
         tagger = self._pipeline.get_by_name('tagger')
@@ -379,7 +417,8 @@ class TagWriteTask(task.Task):
         self.debug('set state to NULL')
         task.Task.stop(self)
 
-class SafeRetagTask(task.MultiSeparateTask):
+
+class SafeRetagTask(ctask.LoggableMultiSeparateTask):
     """
     I am a task that retags an encoded file safely in place.
     First of all, if the new tags are the same as the old ones, it doesn't
@@ -403,7 +442,7 @@ class SafeRetagTask(task.MultiSeparateTask):
         assert type(path) is unicode, "path %r is not unicode" % path
 
         task.MultiSeparateTask.__init__(self)
-        
+
         self._path = path
         self._taglist = taglist.copy()
 
@@ -425,7 +464,8 @@ class SafeRetagTask(task.MultiSeparateTask):
                     self.debug('Current tags: %r, new tags: %r',
                         common.tagListToDict(taglist),
                         common.tagListToDict(self._taglist))
-                    assert common.tagListToDict(taglist) != common.tagListToDict(self._taglist)
+                    assert common.tagListToDict(taglist) \
+                        != common.tagListToDict(self._taglist)
                     self.tasks.append(checksum.CRC32Task(self._path))
                     self._fd, self._tmppath = tempfile.mkstemp(
                         dir=os.path.dirname(self._path), suffix=u'.morituri')
@@ -453,10 +493,11 @@ class SafeRetagTask(task.MultiSeparateTask):
                 else:
                     self.debug('failed to update tags, only have %r',
                         common.tagListToDict(self.tasks[4].taglist))
+                    self.debug('difference: %r',
+                        common.tagListDifference(self.tasks[4].taglist,
+                            self._taglist))
                     os.unlink(self._tmppath)
                     e = TypeError("Tags not written")
                     self.setAndRaiseException(e)
-                   
-        task.MultiSeparateTask.stopped(self, taskk)
 
-      
+        task.MultiSeparateTask.stopped(self, taskk)
