@@ -6,17 +6,17 @@
 # Copyright (C) 2009, 2010, 2011 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -25,133 +25,24 @@ Common functionality and class for all programs using morituri.
 """
 
 import os
-import urlparse
+import sys
+import time
 
-from morituri.common import common, log
-from morituri.result import result
+from morituri.common import common, log, musicbrainzngs, cache
 from morituri.program import cdrdao, cdparanoia
 from morituri.image import image
 
-class MusicBrainzException(Exception):
-    def __init__(self, exc):
-        self.args = (exc, )
-        self.exception = exc
-
-class TrackMetadata(object):
-    artist = None
-    title = None
-
-class DiscMetadata(object):
-    """
-    @param release: earliest release date, in YYYY-MM-DD
-    @type  release: unicode
-    """
-    artist = None
-    sortName = None
-    title = None
-    various = False
-    tracks = None
-    release = None
-
-    mbid = None
-    mbidArtist = None
-
-    def __init__(self):
-        self.tracks = []
 
 def filterForPath(text):
     return "-".join(text.split("/"))
 
-def getMetadata(release):
-    """
-    @rtype: L{DiscMetadata}
-    """
-    metadata = DiscMetadata()
-
-    isSingleArtist = release.isSingleArtistRelease()
-    metadata.various = not isSingleArtist
-    metadata.title = release.title
-    # getUniqueName gets disambiguating names like Muse (UK rock band)
-    metadata.artist = release.artist.name
-    metadata.sortName = release.artist.sortName
-    metadata.release = release.getEarliestReleaseDate()
-    metadata.mbid = urlparse.urlparse(release.id)[2].split("/")[-1]
-    metadata.mbidArtist = urlparse.urlparse(release.artist.id)[2].split("/")[-1]
-
-
-    for t in release.tracks:
-        track = TrackMetadata()
-        if isSingleArtist or t.artist == None:
-            track.artist = metadata.artist
-            track.sortName = metadata.sortName
-            track.mbidArtist = metadata.mbidArtist
-        else:
-            # various artists discs can have tracks with no artist
-            track.artist = t.artist and t.artist.name or release.artist.name
-            track.sortName = t.artist.sortName
-            track.mbidArtist = urlparse.urlparse(t.artist.id)[2].split("/")[-1] 
-
-        track.title = t.title
-        track.mbid = urlparse.urlparse(t.id)[2].split("/")[-1]
-        metadata.tracks.append(track)
-
-    return metadata
-
-
-def musicbrainz(discid):
-    """
-    @rtype: list of L{DiscMetadata}
-    """
-    #import musicbrainz2.disc as mbdisc
-    import musicbrainz2.webservice as mbws
-
-    results = []
-
-    # Setup a Query object.
-    service = mbws.WebService()
-    query = mbws.Query(service)
-
-
-    # Query for all discs matching the given DiscID.
-    # FIXME: let mbws.WebServiceError go through for now
-    try:
-        rfilter = mbws.ReleaseFilter(discId=discid)
-        results = query.getReleases(rfilter)
-    except mbws.WebServiceError, e:
-        raise MusicBrainzException(e)
-
-    # No disc matching this DiscID has been found.
-    if len(results) == 0:
-        return None
-
-    # Display the returned results to the user.
-    ret = []
-
-    for result in results:
-        release = result.release
-        log.debug('program', 'result: artist %r, title %r' % (
-            release.artist.getName(), release.title))
-        # The returned release object only contains title and artist, but no
-        # tracks.  Query the web service once again to get all data we need.
-        try:
-            inc = mbws.ReleaseIncludes(artist=True, tracks=True,
-                releaseEvents=True)
-            # Arid - Under the Cold Street Lights has getId() None
-            if release.getId():
-                release = query.getReleaseById(release.getId(), inc)
-        except mbws.WebServiceError, e:
-            raise MusicBrainzException(e)
-
-        ret.append(getMetadata(release))
-
-    return ret
 
 class Program(log.Loggable):
     """
     I maintain program state and functionality.
 
     @ivar metadata:
-    @type metadata: L{DiscMetadata}
+    @type metadata: L{musicbrainz.DiscMetadata}
     @ivar result:   the rip's result
     @type result:   L{result.RipResult}
     @type outdir:   unicode
@@ -163,15 +54,25 @@ class Program(log.Loggable):
     outdir = None
     result = None
 
+    _stdout = None
+
+    def __init__(self, record=False, stdout=sys.stdout):
+        """
+        @param record: whether to record results of API calls for playback.
+        """
+        self._record = record
+        self._cache = cache.ResultCache()
+        self._stdout = stdout
+
     def _getTableCachePath(self):
         path = os.path.join(os.path.expanduser('~'), '.morituri', 'cache',
             'table')
         return path
 
-    def _getResultCachePath(self):
-        path = os.path.join(os.path.expanduser('~'), '.morituri', 'cache',
-            'result')
-        return path
+    def setWorkingDirectory(self, workingDirectory):
+        if workingDirectory:
+            self.info('Changing to working directory %s' % workingDirectory)
+            os.chdir(workingDirectory)
 
     def loadDevice(self, device):
         """
@@ -198,7 +99,7 @@ class Program(log.Loggable):
         if device in proc:
             print 'Device %s is mounted, unmounting' % device
             os.system('umount %s' % device)
-        
+
     def getTable(self, runner, cddbdiscid, device):
         """
         Retrieve the Table either from the cache or the drive.
@@ -207,7 +108,7 @@ class Program(log.Loggable):
         """
         path = self._getTableCachePath()
 
-        pcache = common.PersistedCache(path)
+        pcache = cache.PersistedCache(path)
         ptable = pcache.get(cddbdiscid)
 
         if not ptable.object:
@@ -221,6 +122,8 @@ class Program(log.Loggable):
 
         return itable
 
+    # FIXME: the cache should be model/offset specific
+
     def getRipResult(self, cddbdiscid):
         """
         Retrieve the persistable RipResult either from our cache (from a
@@ -230,24 +133,15 @@ class Program(log.Loggable):
         """
         assert self.result is None
 
-        path = self._getResultCachePath()
-
-        pcache = common.PersistedCache(path)
-        presult = pcache.get(cddbdiscid)
-
-        if not presult.object:
-            presult.object = result.RipResult()
-            presult.persist(self.result)
-
-        self.result = presult.object
-        self._presult = presult
+        self._presult = self._cache.getRipResult(cddbdiscid)
+        self.result = self._presult.object
 
         return self.result
 
     def saveRipResult(self):
         self._presult.persist()
 
-    def getPath(self, outdir, template, mbdiscid, i):
+    def getPath(self, outdir, template, mbdiscid, i, profile=None):
         """
         Based on the template, get a complete path for the given track,
         minus extension.
@@ -257,15 +151,17 @@ class Program(log.Loggable):
         @type  outdir:   unicode
         @param template: the template for writing the file
         @type  template: unicode
-        @param i:        track number (0 for HTOA)
+        @param i:        track number (0 for HTOA, or for disc)
         @type  i:        int
+        @type  profile:  L{morituri.common.encode.Profile}
 
         @rtype: unicode
         """
         assert type(outdir) is unicode, "%r is not unicode" % outdir
         assert type(template) is unicode, "%r is not unicode" % template
 
-        # the template is similar to grip, except for %s/%S
+        # the template is similar to grip, except for %s/%S/%r/%R
+        # see #gripswitches
 
         # returns without extension
 
@@ -275,7 +171,10 @@ class Program(log.Loggable):
 
         # default values
         v['A'] = 'Unknown Artist'
-        v['d'] = mbdiscid
+        v['d'] = mbdiscid # fallback for title
+        v['r'] = 'unknown'
+        v['R'] = 'Unknown'
+        v['x'] = profile and profile.extension or 'unknown'
 
         v['a'] = v['A']
         if i == 0:
@@ -285,13 +184,19 @@ class Program(log.Loggable):
 
 
         if self.metadata:
+            release = self.metadata.release or '0000'
+            v['y'] = release[:4]
             v['A'] = filterForPath(self.metadata.artist)
             v['S'] = filterForPath(self.metadata.sortName)
             v['d'] = filterForPath(self.metadata.title)
+            if self.metadata.releaseType:
+                v['R'] = self.metadata.releaseType
+                v['r'] = self.metadata.releaseType.lower()
             if i > 0:
                 try:
                     v['a'] = filterForPath(self.metadata.tracks[i - 1].artist)
-                    v['s'] = filterForPath(self.metadata.tracks[i - 1].sortName)
+                    v['s'] = filterForPath(
+                        self.metadata.tracks[i - 1].sortName)
                     v['n'] = filterForPath(self.metadata.tracks[i - 1].title)
                 except IndexError, e:
                     print 'ERROR: no track %d found, %r' % (i, e)
@@ -313,42 +218,117 @@ class Program(log.Loggable):
         """
         # FIXME: convert to nonblocking?
         import CDDB
-        code, md = CDDB.query(cddbdiscid) 
+        code, md = CDDB.query(cddbdiscid)
         self.debug('CDDB query result: %r, %r', code, md)
         if code == 200:
             return md['title']
 
         return None
 
-
-    def getMusicBrainz(self, ittoc, mbdiscid):
+    def getMusicBrainz(self, ittoc, mbdiscid, release=None):
         # look up disc on musicbrainz
+        self._stdout.write('Disc duration: %s\n' % common.formatTime(
+            ittoc.duration() / 1000.0))
+        self.debug('MusicBrainz submit url: %r',
+            ittoc.getMusicBrainzSubmitURL())
         ret = None
 
         metadatas = None
-        try:
-            metadatas = musicbrainz(mbdiscid)
-        except MusicBrainzException, e:
-            print "Error:", e
-            print 'Continuing without metadata'
+        e = None
+
+        for _ in range(0, 4):
+            try:
+                metadatas = musicbrainzngs.musicbrainz(mbdiscid,
+                    record=self._record)
+            except musicbrainzngs.NotFoundException, e:
+                break
+            except musicbrainzngs.MusicBrainzException, e:
+                self._stdout.write("Warning: %r\n" % (e, ))
+                time.sleep(5)
+                continue
+
+        if not metadatas:
+            if e:
+                self._stdout.write("Error: %r\n" % (e, ))
+            self._stdout.write('Continuing without metadata\n')
 
         if metadatas:
-            print 'Matching releases:'
+            deltas = {}
+
+            self._stdout.write('\nMatching releases:\n')
+
             for metadata in metadatas:
-                print 'Artist  : %s' % metadata.artist.encode('utf-8')
-                print 'Title   : %s' % metadata.title.encode('utf-8')
+                self._stdout.write('Artist  : %s\n' %
+                    metadata.artist.encode('utf-8'))
+                self._stdout.write('Title   : %s\n' %
+                    metadata.title.encode('utf-8'))
+                self._stdout.write('Duration: %s\n' %
+                    common.formatTime(metadata.duration / 1000.0))
+                self._stdout.write('URL     : %s\n' % metadata.url)
+                self._stdout.write('Release : %s\n' % metadata.mbid)
+                self._stdout.write('Type    : %s\n' % metadata.releaseType)
+
+                delta = abs(metadata.duration - ittoc.duration())
+                if not delta in deltas:
+                    deltas[delta] = []
+                deltas[delta].append(metadata)
+
+            if release:
+                metadatas = [m for m in metadatas if m.url.endswith(release)]
+                self.debug('Asked for release %r, only kept %r',
+                    release, metadatas)
+                if len(metadatas) == 1:
+                    self._stdout.write('\n')
+                    self._stdout.write('Picked requested release id %s\n' %
+                        release)
+                    self._stdout.write('Artist : %s\n' %
+                        metadatas[0].artist.encode('utf-8'))
+                    self._stdout.write('Title :  %s\n' %
+                        metadatas[0].title.encode('utf-8'))
+                elif not metadatas:
+                    self._stdout.write(
+                        'Requested release id %s but none match' % release)
+                    return
+            else:
+                # Select the release that most closely matches the duration.
+                lowest = min(deltas.keys())
+
+                # If we have multiple, make sure they match
+                metadatas = deltas[lowest]
+
+            if len(metadatas) > 1:
+                artist = metadatas[0].artist
+                releaseTitle = metadatas[0].releaseTitle
+                for i, metadata in enumerate(metadatas):
+                    if not artist == metadata.artist:
+                        self.warning("artist 0: %r and artist %d: %r "
+                            "are not the same" % (
+                                artist, i, metadata.artist))
+                    if not releaseTitle == metadata.releaseTitle:
+                        self.warning("title 0: %r and title %d: %r "
+                            "are not the same" % (
+                                releaseTitle, i, metadata.releaseTitle))
+
+                if (not release and len(deltas.keys()) > 1):
+                    self._stdout.write('\n')
+                    self._stdout.write('Picked closest match in duration.\n')
+                    self._stdout.write('Others may be wrong in musicbrainz, '
+                        'please correct.\n')
+                    self._stdout.write('Artist : %s\n' %
+                        artist.encode('utf-8'))
+                    self._stdout.write('Title :  %s\n' %
+                        metadatas[0].title.encode('utf-8'))
 
             # Select one of the returned releases. We just pick the first one.
             ret = metadatas[0]
         else:
-            print 'Submit this disc to MusicBrainz at:'
-            print ittoc.getMusicBrainzSubmitURL()
+            self._stdout.write(
+                'Submit this disc to MusicBrainz at the above URL.\n')
             ret = None
 
-        print
+        self._stdout.write('\n')
         return ret
 
-
     def getTagList(self, number):
         """
         Based on the metadata, get a gst.TagList for the given track.
@@ -372,10 +352,11 @@ class Program(log.Loggable):
 
             if number > 0:
                 try:
-                    trackArtist = self.metadata.tracks[number - 1].artist
-                    title = self.metadata.tracks[number - 1].title
-                    mbidTrack = self.metadata.tracks[number - 1].mbid
-                    mbidTrackArtist = self.metadata.tracks[number - 1].mbidArtist
+                    track = self.metadata.tracks[number - 1]
+                    trackArtist = track.artist
+                    title = track.title
+                    mbidTrack = track.mbid
+                    mbidTrackArtist = track.mbidArtist
                 except IndexError, e:
                     print 'ERROR: no track %d found, %r' % (number, e)
                     raise
@@ -388,9 +369,10 @@ class Program(log.Loggable):
 
         ret = gst.TagList()
 
-        # gst-python 0.10.15.1 does not handle unicode -> utf8 string conversion
+        # gst-python 0.10.15.1 does not handle unicode -> utf8 string
+        # conversion
         # see http://bugzilla.gnome.org/show_bug.cgi?id=584445
-        if self.metadata.various:
+        if self.metadata and self.metadata.various:
             ret["album-artist"] = albumArtist.encode('utf-8')
         ret[gst.TAG_ARTIST] = trackArtist.encode('utf-8')
         ret[gst.TAG_TITLE] = title.encode('utf-8')
@@ -409,8 +391,8 @@ class Program(log.Loggable):
             # 0.10.15.1
             # FIXME: The dates are strings and must have the format 'YYYY',
             # 'YYYY-MM' or 'YYYY-MM-DD'.
-            # GstDate expects a full date, so default to Jan and 1st if MM and DD
-            # are missing
+            # GstDate expects a full date, so default to
+            # Jan and 1st if MM and DD are missing
             date = self.metadata.release
             if date:
                 log.debug('metadata',
@@ -423,15 +405,15 @@ class Program(log.Loggable):
                 s = gst.structure_from_string('hi,date=(GstDate)%s' %
                     str(date))
                 ret[gst.TAG_DATE] = s['date']
-            
+
             # no musicbrainz info for htoa tracks
             if number > 0:
                 ret["musicbrainz-trackid"] = mbidTrack
                 ret["musicbrainz-artistid"] = mbidTrackArtist
                 ret["musicbrainz-albumid"] = mbidAlbum
                 ret["musicbrainz-albumartistid"] = mbidTrackAlbum
-            
-        # FIXME: gst.TAG_ISRC 
+
+        # FIXME: gst.TAG_ISRC
 
         return ret
 
@@ -459,11 +441,18 @@ class Program(log.Loggable):
 
         runner.run(t)
 
-        return trackResult.testcrc == t.checksum
+        ret = trackResult.testcrc == t.checksum
+        log.debug('program',
+            'verifyTrack: track result crc %r, file crc %r, result %r',
+            trackResult.testcrc, t.checksum, ret)
+        return ret
 
     def ripTrack(self, runner, trackResult, offset, device, profile, taglist,
         what=None):
         """
+        Ripping the track may change the track's filename as stored in
+        trackResult.
+
         @param trackResult: the object to store information in.
         @type  trackResult: L{result.TrackResult}
         @param number:      track number (1-based)
@@ -492,10 +481,24 @@ class Program(log.Loggable):
 
         runner.run(t)
 
+        self.debug('ripped track')
+        self.debug('test speed %.3f/%.3f seconds' % (
+            t.testspeed, t.testduration))
+        self.debug('copy speed %.3f/%.3f seconds' % (
+            t.copyspeed, t.copyduration))
         trackResult.testcrc = t.testchecksum
         trackResult.copycrc = t.copychecksum
         trackResult.peak = t.peak
         trackResult.quality = t.quality
+        trackResult.testspeed = t.testspeed
+        trackResult.copyspeed = t.copyspeed
+        # we want rerips to add cumulatively to the time
+        trackResult.testduration += t.testduration
+        trackResult.copyduration += t.copyduration
+
+        if trackResult.filename != t.path:
+            trackResult.filename = t.path
+            self.info('Filename changed to %r', trackResult.filename)
 
     def retagImage(self, runner, taglists):
         cueImage = image.Image(self.cuePath)
@@ -606,7 +609,8 @@ class Program(log.Loggable):
             # htoa tracks (i == 0) do not have an ARCRC
             if trackResult.ARCRC is None:
                 assert trackResult.number == 0, \
-                    'no trackResult.ARCRC on non-HTOA track'
+                    'no trackResult.ARCRC on non-HTOA track %d' % \
+                        trackResult.number
                 res.append("Track  0: unknown          (not tracked)")
             else:
                 res.append("Track %2d: %s %s [%08x]%s" % (
@@ -620,7 +624,7 @@ class Program(log.Loggable):
         self.debug('write .cue file to %s', cuePath)
         handle = open(cuePath, 'w')
         # FIXME: do we always want utf-8 ?
-        handle.write(self.result.table.cue().encode('utf-8'))
+        handle.write(self.result.table.cue(cuePath).encode('utf-8'))
         handle.close()
 
         self.cuePath = cuePath
