@@ -19,6 +19,7 @@ class CommandHelpFormatter(optparse.IndentedHelpFormatter):
 
     _commands = None
     _aliases = None
+    _klass = None
 
     def addCommand(self, name, description):
         if self._commands is None:
@@ -30,6 +31,29 @@ class CommandHelpFormatter(optparse.IndentedHelpFormatter):
             self._aliases = []
         self._aliases.append(alias)
 
+    def setClass(self, klass):
+        self._klass = klass
+
+    def getCommands(self):
+        ret = ""
+
+        if self._commands:
+            commandDesc = []
+            commandDesc.append("Commands:")
+            keys = self._commands.keys()
+            keys.sort()
+            length = 0
+            for key in keys:
+                if len(key) > length:
+                    length = len(key)
+            for name in keys:
+                formatString = "  %-" + "%d" % length + "s  %s"
+                commandDesc.append(formatString % (name, self._commands[name]))
+            ret += "\n" + "\n".join(commandDesc) + "\n"
+
+        return ret
+
+
     ### override parent method
 
     def format_description(self, description, width=None):
@@ -56,19 +80,12 @@ class CommandHelpFormatter(optparse.IndentedHelpFormatter):
             ret += "\nAliases: " + ", ".join(self._aliases) + "\n"
 
         # add subcommands
-        if self._commands:
-            commandDesc = []
-            commandDesc.append("Commands:")
-            keys = self._commands.keys()
-            keys.sort()
-            length = 0
-            for key in keys:
-                if len(key) > length:
-                    length = len(key)
-            for name in keys:
-                formatString = "  %-" + "%d" % length + "s  %s"
-                commandDesc.append(formatString % (name, self._commands[name]))
-            ret += "\n" + "\n".join(commandDesc) + "\n"
+        ret += self.getCommands()
+
+        # add class info
+        ret += "\nImplemented by: %s.%s\n" % (
+            self._klass.__module__, self._klass.__name__)
+
         return ret
 
 
@@ -99,6 +116,11 @@ class CommandOptionParser(optparse.OptionParser):
     # the signature from the base class
     __pychecker__ = 'no-shadowbuiltin'
 
+    def print_commands(self, file=None):
+        if file is None:
+            file = self._stdout
+        file.write(self.formatter.getCommands())
+
     def print_help(self, file=None):
         # we are overriding a parent method so we can't do anything about file
         __pychecker__ = 'no-shadowbuiltin'
@@ -118,7 +140,7 @@ class CommandOptionParser(optparse.OptionParser):
         return status
 
 
-class Command:
+class Command(object):
     """
     I am a class that handles a command for a program.
     Commands can be nested underneath a command for further processing.
@@ -158,7 +180,7 @@ class Command:
         This redirection will be passed on to child commands.
         """
         if not self.name:
-            self.name = str(self.__class__).split('.')[-1].lower()
+            self.name = self.__class__.__name__.lower()
         self._stdout = stdout
         self._stderr = stderr
         self.parentCommand = parentCommand
@@ -176,10 +198,19 @@ class Command:
 
         # create our formatter and add subcommands if we have them
         formatter = CommandHelpFormatter(width=width)
+        formatter.setClass(self.__class__)
         if self.subCommands:
+            if not self.description:
+                if self.summary:
+                    self.description = self.summary
+                else:
+                    raise AttributeError, \
+                        "%r needs a summary or description " \
+                        "for help formatting" % self
+
             for name, command in self.subCommands.items():
                 formatter.addCommand(name, command.summary or
-                    command.description)
+                    command.description or '')
 
         if self.aliases:
             for alias in self.aliases:
@@ -197,9 +228,15 @@ class Command:
         if not usage.startswith('%prog'):
             usage = self.name + ' ' + usage
 
-        if usage.find("%command") > -1:
-            usage = usage.split("%command")[0] + '[command]'
         usages = [usage, ]
+        if usage.find("%command") > -1:
+            if self.subCommands:
+                usage = usage.split("%command")[0] + '[command]'
+                usages = [usage, ]
+            else:
+                # %command used in a leaf command
+                usages = usage.split("%command")
+                usages.reverse()
 
         # FIXME: abstract this into getUsage that takes an optional
         # parentCommand on where to stop recursing up
@@ -258,7 +295,7 @@ class Command:
         """
         # note: no arguments should be passed as an empty list, not a list
         # with an empty str as ''.split(' ') returns
-        self.debug('calling %r.parse_args' % self)
+        self.debug('calling %r.parse_args(%r)' % (self, argv))
         self.options, args = self.parser.parse_args(argv)
         self.debug('called %r.parse_args' % self)
 
@@ -268,7 +305,9 @@ class Command:
 
         # FIXME: make handleOptions not take options, since we store it
         # in self.options now
+        self.debug('calling %r.handleOptions(%r)' % (self, self.options))
         ret = self.handleOptions(self.options)
+        self.debug('called %r.handleOptions, returned %r' % (self, ret))
         if ret:
             return ret
 
@@ -278,6 +317,9 @@ class Command:
 
             # give help on current command if only 'help' is passed
             if len(args) == 1:
+                # start on a newline for the case where we're in the
+                # interpreter
+                self.stdout.write('\n')
                 self.outputHelp()
                 return 0
 
@@ -299,20 +341,28 @@ class Command:
         # defer to our do() method
         # allows implementing a do() for commands that also have subcommands
         if not args or not self.subCommands:
-            self.debug('no args or no subcommands, doing')
+            self.debug('no args or no subcommands, calling %r.do(%r)' % (
+                self, args))
             try:
                 ret = self.do(args)
+                self.debug('done ok, returned %r', ret)
             except CommandOk, e:
+                self.debug('done with exception, raised %r', e)
                 ret = e.status
-                self.stdout.write(e.output + '\n')
+                if e.output is not None:
+                    self.stdout.write(e.output + '\n')
             except CommandExited, e:
+                self.debug('done with exception, raised %r', e)
                 ret = e.status
-                self.stderr.write(e.output + '\n')
+                if e.output is not None:
+                    self.stderr.write(e.output + '\n')
             except NotImplementedError:
+                self.debug('done with NotImplementedError')
                 self.parser.print_usage(file=self.stderr)
                 self.stderr.write(
                     "Use --help to get a list of commands.\n")
-                return 1
+                ret = 1
+
 
             # if everything's fine, we return 0
             if not ret:
@@ -338,8 +388,12 @@ class Command:
             if command in self.aliasedSubCommands.keys():
                 return self.aliasedSubCommands[command].parse(args[1:])
 
-        self.stderr.write("Unknown command '%s'.\n" % command.encode('utf-8'))
-        self.parser.print_usage(file=self.stderr)
+        if not command:
+            self.stderr.write("Please specify a subcommand.\n")
+        else:
+            self.stderr.write("Unknown command '%s'.\n"
+                % command.encode('utf-8'))
+            self.parser.print_commands(file=self.stderr)
         return 1
 
     def handleOptions(self, options):
@@ -378,6 +432,19 @@ class Command:
             c = c.parentCommand
         return c
 
+    def warning(self, format, *args):
+        """
+        Override me to handle warning output from this class.
+        """
+        pass
+
+    def info(self, format, *args):
+        """
+        Override me to handle info output from this class.
+        """
+        pass
+
+
     def debug(self, format, *args):
         """
         Override me to handle debug output from this class.
@@ -393,25 +460,32 @@ class Command:
         names.reverse()
         return " ".join(names)
 
-    def _getStdout(self):
-        # if set explicitly, use it
-        if self._stdout:
-            return self._stdout
+    def _getStd(self, what):
+
+        ret = getattr(self, '_' + what, None)
+        if ret:
+            return ret
 
         # if I am the root command, default
         if not self.parentCommand:
-            return sys.stdout
+            return getattr(sys, what)
 
         # otherwise delegate to my parent
-        return self.parentCommand.stdout
+        return getattr(self.parentCommand, what)
+
+    def _getStdOut(self):
+        return self._getStd('stdout')
+
+    def _getStdErr(self):
+        return self._getStd('stderr')
+
+    stdout = property(_getStdOut)
+    stderr = property(_getStdErr)
 
-    stdout = property(_getStdout)
-    # FIXME: do we want a separate one ?
-    stderr = property(_getStdout)
 
 class CommandExited(Exception):
 
-    def __init__(self, status, output):
+    def __init__(self, status, output=None):
         self.args = (status, output)
         self.status = status
         self.output = output
@@ -419,13 +493,13 @@ class CommandExited(Exception):
 
 class CommandOk(CommandExited):
 
-    def __init__(self, output):
+    def __init__(self, output=None):
         CommandExited.__init__(self, 0, output)
 
 
 class CommandError(CommandExited):
 
-    def __init__(self, output):
+    def __init__(self, output=None):
         CommandExited.__init__(self, 3, output)
 
 
@@ -512,14 +586,21 @@ def commandToCmdClass(command):
         def generateHelp(c):
 
             def help_(s):
-                command.parser.print_help(file=s.stdout)
+                # add a newline because we're still at the end of the help
+                # command on the prompt
+                c.debug('Getting help for %r', s)
+                s.stdout.write('\n')
+                c.parser.print_help(file=s.stdout)
             return help_
 
         method = generateHelp(subCommand)
+        command.debug('Adding method %r with name %r to %r' % (
+            method, methodName, cmdClass))
         setattr(cmdClass, methodName, method)
 
     return cmdClass
 
+
 def commandToCmd(command):
     # for compatibility reasons
     return commandToCmdClass(command)()
