@@ -848,7 +848,7 @@ class Loggable:
             if hasattr(self, name):
                 return getattr(self, name)
 
-        return '<' + self.__class__.__name__ + '>'
+        return None
 
 # Twisted helper stuff
 
@@ -882,7 +882,13 @@ def getFailureMessage(failure):
     if len(failure.frames) == 0:
         return "failure %(exc)s: %(msg)s" % locals()
 
-    (func, filename, line, some, other) = failure.frames[-1]
+    # when using inlineCallbacks, a traceback coming from unwindGenerator
+    # is actually provoked one frame down
+    for frame in failure.frames[::-1]:
+        (func, filename, line, some, other) = frame
+        if func not in ['unwindGenerator', ]:
+            break
+
     filename = scrubFilename(filename)
     return "failure %(exc)s at %(filename)s:%(line)s: %(func)s(): %(msg)s" \
         % locals()
@@ -993,11 +999,11 @@ class TwistedLogObserver(Loggable):
 
 
 def logLevelToStdLevel(level):
-    t = {ERROR : logging.CRITICAL,
-         WARN : logging.ERROR,
-         INFO : logging.WARNING,
-         DEBUG : logging.INFO,
-         LOG : logging.DEBUG}
+    t = {ERROR: logging.CRITICAL,
+         WARN: logging.ERROR,
+         INFO: logging.WARNING,
+         DEBUG: logging.INFO,
+         LOG: logging.DEBUG}
     return t.get(level, logging.NOTSET)
 
 
