@@ -6,17 +6,17 @@
 # Copyright (C) 2009 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -26,9 +26,12 @@ Wrap on-disk CD images based on the .cue file.
 
 import os
 
-from morituri.common import task, log, common, gstreamer
+from morituri.common import log, common
 from morituri.image import cue, table
 
+from morituri.extern.task import task, gstreamer
+
+
 class Image(object, log.Loggable):
     """
     @ivar table: The Table of Contents for this image.
@@ -42,7 +45,7 @@ class Image(object, log.Loggable):
         @param path: .cue path
         """
         assert type(path) is unicode, "%r is not unicode" % path
-        
+
         self._path = path
         self.cue = cue.CueFile(path)
         self.cue.parse()
@@ -88,7 +91,8 @@ class Image(object, log.Loggable):
             tracks.append(t)
             # FIXME: this probably only works for non-compliant .CUE files
             # where pregap is put at end of previous file
-            t.index(1, absolute=offset, path=self.cue.table.tracks[i].getIndex(1).path,
+            t.index(1, absolute=offset,
+                path=self.cue.table.tracks[i].getIndex(1).path,
                 relative=0)
 
             offset += length
@@ -98,11 +102,11 @@ class Image(object, log.Loggable):
         self.debug('setup image done')
 
 
-class AccurateRipChecksumTask(task.MultiSeparateTask):
+class AccurateRipChecksumTask(log.Loggable, task.MultiSeparateTask):
     """
     I calculate the AccurateRip checksums of all tracks.
     """
-    
+
     description = "Checksumming tracks"
 
     def __init__(self, image):
@@ -116,7 +120,11 @@ class AccurateRipChecksumTask(task.MultiSeparateTask):
         for trackIndex, track in enumerate(cue.table.tracks):
             index = track.indexes[1]
             length = cue.getTrackLength(track)
-            self.debug('track %d has length %d' % (trackIndex + 1, length))
+            if length < 0:
+                self.debug('track %d has unknown length' % (trackIndex + 1, ))
+            else:
+                self.debug('track %d is %d samples long' % (
+                    trackIndex + 1, length))
 
             path = image.getRealPath(index.path)
 
@@ -125,19 +133,20 @@ class AccurateRipChecksumTask(task.MultiSeparateTask):
 
             checksumTask = checksum.AccurateRipChecksumTask(path,
                 trackNumber=trackIndex + 1, trackCount=len(cue.table.tracks),
-                frameStart=index.relative * common.SAMPLES_PER_FRAME,
-                frameLength=length * common.SAMPLES_PER_FRAME)
+                sampleStart=index.relative * common.SAMPLES_PER_FRAME,
+                sampleLength=length * common.SAMPLES_PER_FRAME)
             self.addTask(checksumTask)
 
     def stop(self):
         self.checksums = [t.checksum for t in self.tasks]
         task.MultiSeparateTask.stop(self)
 
-class AudioLengthTask(gstreamer.GstPipelineTask):
-    """
-    I calculate the length of a track in audio frames.
 
-    @ivar  length: length of the decoded audio file, in audio frames.
+class AudioLengthTask(log.Loggable, gstreamer.GstPipelineTask):
+    """
+    I calculate the length of a track in audio samples.
+
+    @ivar  length: length of the decoded audio file, in audio samples.
     """
     logCategory = 'AudioLengthTask'
     description = 'Getting length of audio track'
@@ -145,7 +154,6 @@ class AudioLengthTask(gstreamer.GstPipelineTask):
 
     playing = False
 
-
     def __init__(self, path):
         """
         @type  path: unicode
@@ -153,13 +161,14 @@ class AudioLengthTask(gstreamer.GstPipelineTask):
         assert type(path) is unicode, "%r is not unicode" % path
 
         self._path = path
+        self.logName = os.path.basename(path).encode('utf-8')
 
     def getPipelineDesc(self):
         return '''
             filesrc location="%s" !
             decodebin ! audio/x-raw-int !
             fakesink name=sink''' % \
-                common.quoteParse(self._path).encode('utf-8')
+                gstreamer.quoteParse(self._path).encode('utf-8')
 
     def paused(self):
         self.debug('query duration')
@@ -183,11 +192,12 @@ class AudioLengthTask(gstreamer.GstPipelineTask):
         self.pipeline.set_state(self.gst.STATE_NULL)
         self.stop()
 
-class ImageVerifyTask(task.MultiSeparateTask):
+
+class ImageVerifyTask(log.Loggable, task.MultiSeparateTask):
     """
     I verify a disk image and get the necessary track lengths.
     """
-    
+
     logCategory = 'ImageVerifyTask'
 
     description = "Checking tracks"
@@ -232,11 +242,12 @@ class ImageVerifyTask(task.MultiSeparateTask):
 
         task.MultiSeparateTask.stop(self)
 
-class ImageEncodeTask(task.MultiSeparateTask):
+
+class ImageEncodeTask(log.Loggable, task.MultiSeparateTask):
     """
     I encode a disk image to a different format.
     """
-    
+
     description = "Encoding tracks"
 
     def __init__(self, image, profile, outdir):
