@@ -6,17 +6,17 @@
 # Copyright (C) 2009 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -25,7 +25,6 @@ Wrap Table of Contents.
 """
 
 import copy
-import os
 import urllib
 import urlparse
 
@@ -117,6 +116,7 @@ class Track:
 
         return self.indexes[1].absolute - self.indexes[0].absolute
 
+
 class Index:
     """
     @ivar counter: counter for the index source; distinguishes between
@@ -142,9 +142,10 @@ class Index:
         self.counter = counter
 
     def __repr__(self):
-        return '<Index %02d, absolute %r, path %r, relative %r, counter %r>' % (
+        return '<Index %02d absolute %r path %r relative %r counter %r>' % (
             self.number, self.absolute, self.path, self.relative, self.counter)
 
+
 class Table(object, log.Loggable):
     """
     I represent a table of indexes on a CD.
@@ -232,7 +233,6 @@ class Table(object, log.Loggable):
         """
         return len([t for t in self.tracks if not t.audio]) > 0
 
-
     def _cddbSum(self, i):
         ret = 0
         while i > 0:
@@ -241,54 +241,6 @@ class Table(object, log.Loggable):
 
         return ret
 
-    def _getCDDBValues(self):
-        """
-        Get all CDDB values needed to calculate disc id and lookup URL.
-
-        This includes:
-         - CDDB disc id
-         - number of audio tracks
-         - offset of index 1 of each track
-         - length of disc in seconds
-
-        @rtype:   list of int
-        """
-        result = []
-
-        # number of first track
-        result.append(1)
-
-        # number of last audio track
-        result.append(self.getAudioTracks())
-
-        leadout = self.leadout
-        # if the disc is multi-session, last track is the data track,
-        # and we should subtract 11250 + 150 from the last track's offset
-        # for the leadout
-        if self.hasDataTracks():
-            assert not self.tracks[-1].audio
-            leadout = self.tracks[-1].getIndex(1).absolute - 11250 - 150
-
-        # treat leadout offset as track 0 offset
-        result.append(150 + leadout)
-
-        # offsets of tracks
-        for i in range(1, 100):
-            try:
-                track = self.tracks[i - 1]
-                if not track.audio:
-                    continue
-                offset = track.getIndex(1).absolute + 150
-                result.append(offset)
-            except IndexError:
-                pass
-
-
-        self.debug('CDDB values: %r', result)
-        return result
-
-
-
     def getCDDBValues(self):
         """
         Get all CDDB values needed to calculate disc id and lookup URL.
@@ -297,7 +249,7 @@ class Table(object, log.Loggable):
          - CDDB disc id
          - number of audio tracks
          - offset of index 1 of each track
-         - length of disc in seconds
+         - length of disc in seconds (including data track)
 
         @rtype:   list of int
         """
@@ -323,14 +275,22 @@ class Table(object, log.Loggable):
             seconds = offset / common.FRAMES_PER_SECOND
             n += self._cddbSum(seconds)
 
-        last = self.tracks[-1]
         # the 'real' leadout, not offset by 150 frames
         # print 'THOMAS: disc leadout', self.leadout
+        last = self.tracks[-1]
         leadout = self.getTrackEnd(last.number) + 1
         self.debug('leadout LBA: %d', leadout)
+
+        # FIXME: we can't replace these calculations with the getFrameLength
+        # call because the start and leadout in the algorithm get rounded
+        # before making the difference
         startSeconds = self.getTrackStart(1) / common.FRAMES_PER_SECOND
         leadoutSeconds = leadout / common.FRAMES_PER_SECOND
         t = leadoutSeconds - startSeconds
+        # durationFrames = self.getFrameLength(data=True)
+        # duration = durationFrames / common.FRAMES_PER_SECOND
+        # assert t == duration, "%r != %r" % (t, duration)
+
         debug.append(str(leadoutSeconds + 2)) # 2 is the 150 frame cddb offset
         result.append(leadoutSeconds)
 
@@ -342,9 +302,8 @@ class Table(object, log.Loggable):
 
         self.debug('cddb disc id debug: %s',
             " ".join(["%08x" % value, ] + debug))
-        
-        return result
 
+        return result
 
     def getCDDBDiscId(self):
         """
@@ -356,7 +315,6 @@ class Table(object, log.Loggable):
         values = self.getCDDBValues()
         return "%08x" % values[0]
 
-
     def getMusicBrainzDiscId(self):
         """
         Calculate the MusicBrainz disc ID.
@@ -421,12 +379,35 @@ class Table(object, log.Loggable):
         query = urllib.urlencode({
             'id': discid,
             'toc': ' '.join([str(v) for v in values]),
-            'tracks': self.getAudioTracks()
+            'tracks': self.getAudioTracks(),
         })
 
         return urlparse.urlunparse((
             'http', host, '/bare/cdlookup.html', '', query, ''))
 
+    def getFrameLength(self, data=False):
+        """
+        Get the length in frames (excluding HTOA)
+
+        @param data: whether to include the data tracks in the length
+        """
+        # the 'real' leadout, not offset by 150 frames
+        if data:
+            last = self.tracks[-1]
+        else:
+            last = self.tracks[self.getAudioTracks() - 1]
+
+        leadout = self.getTrackEnd(last.number) + 1
+        self.debug('leadout LBA: %d', leadout)
+        durationFrames = leadout - self.getTrackStart(1)
+
+        return durationFrames
+
+    def duration(self):
+        """
+        Get the duration in ms for all audio tracks (excluding HTOA).
+        """
+        return int(self.getFrameLength() * 1000.0 / common.FRAMES_PER_SECOND)
 
     def _getMusicBrainzValues(self):
         """
@@ -476,7 +457,6 @@ class Table(object, log.Loggable):
         self.debug('Musicbrainz values: %r', result)
         return result
 
-
     def getAccurateRipIds(self):
         """
         Calculate the two AccurateRip ID's.
@@ -518,25 +498,29 @@ class Table(object, log.Loggable):
         discId1, discId2 = self.getAccurateRipIds()
 
         return "http://www.accuraterip.com/accuraterip/" \
-            "%s/%s/%s/dBAR-%.3d-%s-%s-%s.bin" % ( 
+            "%s/%s/%s/dBAR-%.3d-%s-%s-%s.bin" % (
                 discId1[-1], discId1[-2], discId1[-3],
                 self.getAudioTracks(), discId1, discId2, self.getCDDBDiscId())
 
-    def cue(self, program='Morituri'):
+    def cue(self, cuePath='', program='Morituri'):
         """
+        @param cuePath: path to the cue file to be written. If empty,
+                        will treat paths as if in current directory.
+
+
         Dump our internal representation to a .cue file content.
         """
         lines = []
 
         def writeFile(path):
-            lines.append('FILE "%s" WAVE' % os.path.basename(path))
-
+            targetPath = common.getRelativePath(path, cuePath)
+            lines.append('FILE "%s" WAVE' % targetPath)
 
         # header
         main = ['PERFORMER', 'TITLE']
 
         for key in CDTEXT_FIELDS:
-                if key not in main and self.cdtext.has_key(key):
+                if key not in main and key in self.cdtext:
                     lines.append("    %s %s" % (key, self.cdtext[key]))
 
         assert self.hasTOC(), "Table does not represent a full CD TOC"
@@ -547,7 +531,7 @@ class Table(object, log.Loggable):
             lines.append("CATALOG %s" % self.catalog)
 
         for key in main:
-            if self.cdtext.has_key(key):
+            if key in self.cdtext:
                 lines.append('%s "%s"' % (key, self.cdtext[key]))
 
         # add the first FILE line
@@ -562,14 +546,14 @@ class Table(object, log.Loggable):
 
             # if there is no index 0, but there is a new file, advance
             # FILE line here
-            if not track.indexes.has_key(0):
+            if not 0 in track.indexes:
                 index = track.indexes[1]
                 if index.counter != counter:
                     writeFile(index.path)
                     counter = index.counter
             lines.append("  TRACK %02d %s" % (i + 1, 'AUDIO'))
             for key in CDTEXT_FIELDS:
-                if track.cdtext.has_key(key):
+                if key in track.cdtext:
                     lines.append('    %s "%s"' % (key, track.cdtext[key]))
 
             if track.isrc is not None:
@@ -588,9 +572,10 @@ class Table(object, log.Loggable):
 
         lines.append("")
 
-        return "\n".join(lines) 
+        return "\n".join(lines)
 
     ### methods that modify the table
+
     def clearFiles(self):
         """
         Clear all file backings.
@@ -614,7 +599,6 @@ class Table(object, log.Loggable):
             except IndexError:
                 break
 
-
     def setFile(self, track, index, path, length, counter=None):
         """
         Sets the given file as the source from the given index on.
@@ -651,7 +635,7 @@ class Table(object, log.Loggable):
         """
         Calculate absolute offsets on indexes as much as possible.
         Only possible for as long as tracks draw from the same file.
-        """ 
+        """
         t = self.tracks[0].number
         index = self.tracks[0].getFirstIndex()
         i = index.number
@@ -668,7 +652,7 @@ class Table(object, log.Loggable):
             if index.counter is None:
                 self.debug('Track %d, index %d has no counter', t, i)
                 break
-            if  index.counter != counter:
+            if index.counter != counter:
                 self.debug('Track %d, index %d has a different counter', t, i)
                 break
             self.debug('Setting absolute offset %d on track %d, index %d',
@@ -732,6 +716,7 @@ class Table(object, log.Loggable):
         return gap
 
     ### lookups
+
     def getNextTrackIndex(self, track, index):
         """
         Return the next track and index.
@@ -751,16 +736,16 @@ class Table(object, log.Loggable):
 
         track += 1
         if track > len(self.tracks):
-            raise IndexError, "No index beyond track %d, index %d" % (
-                track - 1, index)
+            raise IndexError("No index beyond track %d, index %d" % (
+                track - 1, index))
 
         t = self.tracks[track - 1]
         indexes = t.indexes.keys()
 
         return track, indexes[0]
 
-
     # various tests for types of Table
+
     def hasTOC(self):
         """
         Check if the Table has a complete TOC.
