@@ -6,29 +6,35 @@
 # Copyright (C) 2009 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
+import errno
+import time
 import re
 import stat
 import shutil
 import subprocess
 import tempfile
 
-from morituri.common import task, log, common
+from morituri.common import log, common
+from morituri.common import task as ctask
+
 from morituri.extern import asyncsub
+from morituri.extern.task import task
+
 
 class FileSizeError(Exception):
 
@@ -37,19 +43,27 @@ class FileSizeError(Exception):
     """
     The given path does not have the expected size.
     """
+
     def __init__(self, path, message):
         self.args = (path, message)
         self.path = path
         self.message = message
 
+
 class ReturnCodeError(Exception):
     """
     The program had a non-zero return code.
     """
+
     def __init__(self, returncode):
         self.args = (returncode, )
         self.returncode = returncode
 
+
+class ChecksumException(Exception):
+    pass
+
+
 _PROGRESS_RE = re.compile(r"""
     ^\#\#: (?P<code>.+)\s      # function code
     \[(?P<function>.*)\]\s@\s     # function name
@@ -61,6 +75,7 @@ _ERROR_RE = re.compile("^scsi_read error:")
 # from reading cdparanoia source code, it looks like offset is reported in
 # number of single-channel samples, ie. 2 bytes per unit, and absolute
 
+
 class ProgressParser(object):
     read = 0 # last [read] frame
     wrote = 0 # last [wrote] frame
@@ -84,7 +99,6 @@ class ProgressParser(object):
 
         self._reads = {} # read count for each sector
 
-
     def parse(self, line):
         """
         Parse a line.
@@ -165,7 +179,7 @@ class ProgressParser(object):
         # cdparanoia outputs most [wrote] calls with one word less than a frame
         frameOffset = (wordOffset + 1) / common.WORDS_PER_FRAME
         self.wrote = frameOffset
-        
+
     def getTrackQuality(self):
         """
         Each frame gets read twice.
@@ -180,7 +194,9 @@ class ProgressParser(object):
 
 
 # FIXME: handle errors
-class ReadTrackTask(task.Task):
+
+
+class ReadTrackTask(log.Loggable, task.Task):
     """
     I am a task that reads a track using cdparanoia.
 
@@ -189,6 +205,8 @@ class ReadTrackTask(task.Task):
 
     description = "Reading track"
     quality = None # set at end of reading
+    speed = None
+    duration = None # in seconds
 
     _MAXERROR = 100 # number of errors detected by parser
 
@@ -223,6 +241,7 @@ class ReadTrackTask(task.Task):
         self._offset = offset
         self._parser = ProgressParser(start, stop)
         self._device = device
+        self._start_time = None
 
         self._buffer = "" # accumulate characters
         self._errors = []
@@ -274,6 +293,7 @@ class ReadTrackTask(task.Task):
 
             raise
 
+        self._start_time = time.time()
         self.schedule(1.0, self._read, runner)
 
     def _read(self, runner):
@@ -325,6 +345,7 @@ class ReadTrackTask(task.Task):
         self._done()
 
     def _done(self):
+        end_time = time.time()
         self.setProgress(1.0)
 
         # check if the length matches
@@ -354,18 +375,31 @@ class ReadTrackTask(task.Task):
                 self.exception = ReturnCodeError(self._popen.returncode)
 
         self.quality = self._parser.getTrackQuality()
-            
+        self.duration = end_time - self._start_time
+        self.speed = (offsetLength / 75.0) / self.duration
+
         self.stop()
         return
 
-class ReadVerifyTrackTask(task.MultiSeparateTask):
+
+class ReadVerifyTrackTask(log.Loggable, task.MultiSeparateTask):
     """
     I am a task that reads and verifies a track using cdparanoia.
+    I also encode the track.
+
+    The path where the file is stored can be changed if necessary, for
+    example if the file name is too long.
 
     @ivar path:         the path where the file is to be stored.
     @ivar checksum:     the checksum of the track; set if they match.
     @ivar testchecksum: the test checksum of the track.
     @ivar copychecksum: the copy checksum of the track.
+    @ivar testspeed:    the test speed of the track, as a multiple of
+                        track duration.
+    @ivar copyspeed:    the copy speed of the track, as a multiple of
+                        track duration.
+    @ivar testduration: the test duration of the track, in seconds.
+    @ivar copyduration: the copy duration of the track, in seconds.
     @ivar peak:         the peak level of the track
     """
 
@@ -374,6 +408,10 @@ class ReadVerifyTrackTask(task.MultiSeparateTask):
     copychecksum = None
     peak = None
     quality = None
+    testspeed = None
+    copyspeed = None
+    testduration = None
+    copyduration = None
 
     _tmpwavpath = None
     _tmppath = None
@@ -445,33 +483,125 @@ class ReadVerifyTrackTask(task.MultiSeparateTask):
         self.file_mode = 0666 - umask
 
     def stop(self):
-        if not self.exception:
-            self.quality = max(self.tasks[0].quality, self.tasks[2].quality)
-            self.peak = self.tasks[4].peak
-            self.debug('peak: %r', self.peak)
+        # FIXME: maybe this kind of try-wrapping to make sure
+        # we chain up should be handled by a parent class function ?
+        try:
+            if not self.exception:
+                self.quality = max(self.tasks[0].quality,
+                    self.tasks[2].quality)
+                self.peak = self.tasks[4].peak
+                self.debug('peak: %r', self.peak)
+                self.testspeed = self.tasks[0].speed
+                self.copyspeed = self.tasks[2].speed
+                self.testduration = self.tasks[0].duration
+                self.copyduration = self.tasks[2].duration
 
-            self.testchecksum = c1 = self.tasks[1].checksum
-            self.copychecksum = c2 = self.tasks[3].checksum
-            if c1 == c2:
-                self.info('Checksums match, %08x' % c1)
-                self.checksum = self.testchecksum
+                self.testchecksum = c1 = self.tasks[1].checksum
+                self.copychecksum = c2 = self.tasks[3].checksum
+                if c1 == c2:
+                    self.info('Checksums match, %08x' % c1)
+                    self.checksum = self.testchecksum
+                else:
+                    # FIXME: detect this before encoding
+                    self.info('Checksums do not match, %08x %08x' % (
+                        c1, c2))
+                    self.exception = ChecksumException(
+                        'read and verify failed: test checksum')
+
+                if self.tasks[5].checksum != self.checksum:
+                    self.exception = ChecksumException(
+                        'Encoding failed, checksum does not match')
+
+                # delete the unencoded file
+                os.unlink(self._tmpwavpath)
+
+                os.chmod(self._tmppath, self.file_mode)
+
+                if not self.exception:
+                    try:
+                        self.debug('Moving to final path %r', self.path)
+                        shutil.move(self._tmppath, self.path)
+                    except IOError, e:
+                        if e.errno == errno.ENAMETOOLONG:
+                            self.path = common.shrinkPath(self.path)
+                            shutil.move(self._tmppath, self.path)
+                    except Exception, e:
+                        self.debug('Exception while moving to final path %r: '
+                            '%r',
+                            self.path, log.getExceptionMessage(e))
+                        self.exception = e
+                else:
+                    os.unlink(self._tmppath)
             else:
-                # FIXME: detect this before encoding
-                self.error('read and verify failed')
-
-            if self.tasks[5].checksum != self.checksum:
-                self.error('Encoding failed, checksum does not match')
-
-            # delete the unencoded file
-            os.unlink(self._tmpwavpath)
-
-            os.chmod(self._tmppath, self.file_mode)
-
-            try:
-                shutil.move(self._tmppath, self.path)
-            except Exception, e:
-                self._exception = e
-        else:
-            self.debug('stop: exception %r', self.exception)
+                self.debug('stop: exception %r', self.exception)
+        except Exception, e:
+            print 'WARNING: unhandled exception %r' % (e, )
 
         task.MultiSeparateTask.stop(self)
+
+_VERSION_RE = re.compile(
+    "^cdparanoia (?P<version>.+) release (?P<release>.+) \(.*\)")
+
+
+def getCdParanoiaVersion():
+    version = "(Unknown)"
+
+    try:
+        p = asyncsub.Popen(["cdparanoia", "-V"],
+                stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE, close_fds=True)
+        version = asyncsub.recv_some(p, e=0, stderr=1)
+        vre = _VERSION_RE.search(version)
+        if vre and len(vre.groups()) == 2:
+            version = "%s %s" % (
+                vre.groupdict().get('version'),
+                vre.groupdict().get('release'))
+    except OSError, e:
+        import errno
+        if e.errno == errno.ENOENT:
+            raise common.MissingDependencyException('cdparanoia')
+        raise
+
+    return version
+
+
+_OK_RE = re.compile(r'Drive tests OK with Paranoia.')
+
+
+class AnalyzeTask(ctask.PopenTask):
+
+    logCategory = 'AnalyzeTask'
+    description = 'Analyzing drive caching behaviour'
+
+    defeatsCache = None
+
+    cwd = None
+
+    _output = []
+
+    def __init__(self, device=None):
+        # cdparanoia -A *always* writes cdparanoia.log
+        self.cwd = tempfile.mkdtemp(suffix='.morituri.cache')
+        self.command = ['cdparanoia', '-A']
+        if device:
+            self.command += ['-d', device]
+        
+    def commandMissing(self):
+        raise common.MissingDependencyException('cdparanoia')
+
+    def readbyteserr(self, bytes):
+        self._output.append(bytes)
+
+    def done(self):
+        if self.cwd:
+            shutil.rmtree(self.cwd)
+        output = "".join(self._output)
+        m = _OK_RE.search(output)
+        if m:
+            self.defeatsCache = True
+        else:
+            self.defeatsCache = False
+
+    def failed(self):
+        if self.cwd:
+            shutil.rmtree(self.cwd)
