@@ -6,35 +6,37 @@
 # Copyright (C) 2009 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
 
 import re
 import os
-import signal
-import subprocess
 import tempfile
 
-from morituri.common import task, log, common
+from morituri.common import log, common
 from morituri.image import toc, table
-from morituri.extern import asyncsub
+from morituri.common import task as ctask
+
+from morituri.extern.task import task
+
 
 class ProgramError(Exception):
     """
     The program had a fatal error.
     """
+
     def __init__(self, errorMessage):
         self.args = (errorMessage, )
         self.errorMessage = errorMessage
@@ -71,11 +73,13 @@ _POSITION_RE = re.compile(r"""
 
 _ERROR_RE = re.compile(r"""^ERROR: (?P<error>.*)""")
 
+
 class LineParser(object, log.Loggable):
     """
     Parse incoming bytes into lines
     Calls 'parse' on owner for each parsed line.
     """
+
     def __init__(self, owner):
         self._buffer = ""     # accumulate characters
         self._lines = []      # accumulate lines
@@ -104,15 +108,17 @@ class LineParser(object, log.Loggable):
 
             self._lines.extend(lines)
 
+
 class OutputParser(object, log.Loggable):
+
     def __init__(self, taskk, session=None):
         self._buffer = ""     # accumulate characters
         self._lines = []      # accumulate lines
         self._state = 'START'
         self._frames = None   # number of frames
-        self._track = None    # which track are we analyzing?
+        self.track = 0        # which track are we analyzing?
         self._task = taskk
-        self._tracks = 0      # count of tracks, relative to session
+        self.tracks = 0      # count of tracks, relative to session
         self._session = session
 
 
@@ -126,7 +132,8 @@ class OutputParser(object, log.Loggable):
         # find counter in LEADOUT state; only when we read full toc
         self.log('state: %s, buffer bytes: %d', self._state, len(self._buffer))
         if self._buffer and self._state == 'LEADOUT':
-            # split on lines that end in \r, which reset cursor to counter start
+            # split on lines that end in \r, which reset cursor to counter
+            # start
             # this misses the first one, but that's ok:
             # length 03:40:71...\n00:01:00
             times = self._buffer.split('\r')
@@ -142,8 +149,8 @@ class OutputParser(object, log.Loggable):
 
             # we need both a position reported and an Analyzing line
             # to have been parsed to report progress
-            if m and self._track is not None:
-                track = self.table.tracks[self._track - 1]
+            if m and self.track is not None:
+                track = self.table.tracks[self.track - 1]
                 frame = (track.getIndex(1).absolute or 0) \
                     + int(m.group('hh')) * 60 * common.FRAMES_PER_SECOND \
                     + int(m.group('mm')) * common.FRAMES_PER_SECOND \
@@ -177,7 +184,6 @@ class OutputParser(object, log.Loggable):
             self._parse(lines)
             self._lines.extend(lines)
 
-
     def _parse(self, lines):
         for line in lines:
             #print 'parsing', len(line), line
@@ -201,7 +207,6 @@ class OutputParser(object, log.Loggable):
             error = m.group('error')
             self._task.errors.append(error)
 
-
     def _parse_TRACK(self, line):
         if line.startswith('---'):
             return
@@ -209,12 +214,12 @@ class OutputParser(object, log.Loggable):
         m = _TRACK_RE.search(line)
         if m:
             t = int(m.group('track'))
-            self._tracks += 1
-            track = table.Track(self._tracks, session=self._session)
+            self.tracks += 1
+            track = table.Track(self.tracks, session=self._session)
             track.index(1, absolute=int(m.group('start')))
             self.table.tracks.append(track)
             self.debug('Found absolute track %d, session-relative %d', t,
-                self._tracks)
+                self.tracks)
 
         m = _LEADOUT_RE.search(line)
         if m:
@@ -222,7 +227,7 @@ class OutputParser(object, log.Loggable):
             self._state = 'LEADOUT'
             self._frames = int(m.group('start'))
             self.debug('Found absolute leadout at offset %r', self._frames)
-            self.info('%d tracks found for this session', self._tracks)
+            self.info('%d tracks found for this session', self.tracks)
             return
 
     def _parse_LEADOUT(self, line):
@@ -231,14 +236,13 @@ class OutputParser(object, log.Loggable):
             self.debug('Found analyzing line')
             track = int(m.group('track'))
             self.description = 'Analyzing track %d...' % track
-            self._track = track
-            #self.setProgress(float(track - 1) / self._tracks)
-            #print 'analyzing', track
+            self.track = track
 
 
 # FIXME: handle errors
 
-class CDRDAOTask(task.Task):
+
+class CDRDAOTask(ctask.PopenTask):
     """
     I am a task base class that runs CDRDAO.
     """
@@ -249,99 +253,24 @@ class CDRDAOTask(task.Task):
 
     def __init__(self):
         self.errors = []
+        self.debug('creating CDRDAOTask')
 
     def start(self, runner):
-        task.Task.start(self, runner)
+        self.debug('Starting cdrdao with options %r', self.options)
+        self.command = ['cdrdao', ] + self.options
 
-        bufsize = 1024
-        try:
-            self._popen = asyncsub.Popen(["cdrdao", ] + self.options,
-                bufsize=bufsize,
-                stdin=subprocess.PIPE, stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE, close_fds=True)
-        except OSError, e:
-            import errno
-            if e.errno == errno.ENOENT:
-                raise common.MissingDependencyException('cdrdao')
+        ctask.PopenTask.start(self, runner)
 
-            raise
+    def commandMissing(self):
+        raise common.MissingDependencyException('cdrdao')
 
-        self.debug('Started cdrdao with pid %d and options %r',
-            self._popen.pid, self.options)
-        self.debug('command: cdrdao %s', ' '.join(self.options))
 
-        self.schedule(1.0, self._read, runner)
+    def failed(self):
+        if self.errors:
+            raise DeviceOpenException("\n".join(self.errors))
+        else:
+            raise ProgramFailedException(self._popen.returncode)
 
-    def _read(self, runner):
-        try:
-            ret = self._popen.recv()
-
-            if ret:
-                self.log("read from stdout: %s", ret)
-                self.readbytesout(ret)
-
-            ret = self._popen.recv_err()
-
-            if ret:
-                self.log("read from stderr: %s", ret)
-                self.readbyteserr(ret)
-
-            if self._popen.poll() is None and self.runner:
-                # not finished yet
-                self.schedule(1.0, self._read, runner)
-                return
-
-            self._done()
-        except Exception, e:
-            self.debug('exception during _read()')
-            self.debug(log.getExceptionMessage(e))
-            self.setException(e)
-            self.stop()
-
-    def _done(self):
-            assert self._popen.returncode is not None, "No returncode"
-
-            if self._popen.returncode >= 0:
-                self.debug('Return code was %d', self._popen.returncode)
-            else:
-                self.debug('Terminated with signal %d',
-                    -self._popen.returncode)
-
-            self.setProgress(1.0)
-
-            if self._popen.returncode != 0:
-                if self.errors:
-                    raise DeviceOpenException("\n".join(self.errors))
-                else:
-                    raise ProgramFailedException(self._popen.returncode)
-            else:
-                self.done()
-
-            self.stop()
-            return
-
-    def abort(self):
-        self.debug('Aborting, sending SIGTERM to %d', self._popen.pid)
-        os.kill(self._popen.pid, signal.SIGTERM)
-        # self.stop()
-
-    def readbytesout(self, bytes):
-        """
-        Called when bytes have been read from stdout.
-        """
-        pass
-
-    def readbyteserr(self, bytes):
-        """
-        Called when bytes have been read from stderr.
-        """
-        pass
-
-    def done(self):
-        """
-        Called when cdrdao completed successfully.
-        """
-        raise NotImplementedError
 
 class DiscInfoTask(CDRDAOTask):
     """
@@ -361,6 +290,7 @@ class DiscInfoTask(CDRDAOTask):
         @param device:  the device to rip from
         @type  device:  str
         """
+        self.debug('creating DiscInfoTask')
         CDRDAOTask.__init__(self)
 
         self.options = ['disk-info', ]
@@ -375,11 +305,10 @@ class DiscInfoTask(CDRDAOTask):
     def readbyteserr(self, bytes):
         self.parser.read(bytes)
 
-
     def parse(self, line):
         # called by parser
         if line.startswith('Sessions'):
-            self.sessions = int (line[line.find(':') + 1:])
+            self.sessions = int(line[line.find(':') + 1:])
             self.debug('Found %d sessions', self.sessions)
         m = _ERROR_RE.search(line)
         if m:
@@ -391,6 +320,8 @@ class DiscInfoTask(CDRDAOTask):
 
 
 # Read stuff for one session
+
+
 class ReadSessionTask(CDRDAOTask):
     """
     I am a task that reads things for one session.
@@ -433,6 +364,9 @@ class ReadSessionTask(CDRDAOTask):
     def readbyteserr(self, bytes):
         self.parser.read(bytes)
 
+        if self.parser.tracks > 0:
+            self.setProgress(float(self.parser.track - 1) / self.parser.tracks)
+
     def done(self):
         # by merging the TOC info.
         self._tocfile = toc.TocFile(self._tocfilepath)
@@ -469,6 +403,7 @@ class ReadTableSessionTask(ReadSessionTask):
     logCategory = 'ReadTableSessionTask'
     description = "Scanning indexes"
 
+
 class ReadTOCSessionTask(ReadSessionTask):
     """
     I am a task that reads the TOC of a CD, without pregaps.
@@ -477,7 +412,7 @@ class ReadTOCSessionTask(ReadSessionTask):
     @type table: L{table.Table}
     """
 
-    logCategory = 'ReadTOCSessionTask'
+    logCategory = 'ReadTOCSessTask'
     description = "Reading TOC"
     extraOptions = ['--fast-toc', ]
 
@@ -487,6 +422,8 @@ class ReadTOCSessionTask(ReadSessionTask):
         assert self.table.hasTOC(), "This Table Index should be a TOC"
 
 # read all sessions
+
+
 class ReadAllSessionsTask(task.MultiSeparateTask):
     """
     I am a base class for tasks that need to read all sessions.
@@ -508,6 +445,7 @@ class ReadAllSessionsTask(task.MultiSeparateTask):
 
         self._device = device
 
+        self.debug('Starting ReadAllSessionsTask')
         self.tasks = [DiscInfoTask(device=device), ]
 
     def stopped(self, taskk):
@@ -541,6 +479,7 @@ class ReadTableTask(ReadAllSessionsTask):
     description = "Scanning indexes..."
     _readClass = ReadTableSessionTask
 
+
 class ReadTOCTask(ReadAllSessionsTask):
     """
     I am a task that reads the TOC of a CD, without pregaps.
@@ -553,13 +492,16 @@ class ReadTOCTask(ReadAllSessionsTask):
     description = "Reading TOC..."
     _readClass = ReadTOCSessionTask
 
+
 class DeviceOpenException(Exception):
+
     def __init__(self, msg):
         self.msg = msg
         self.args = (msg, )
 
+
 class ProgramFailedException(Exception):
+
     def __init__(self, code):
         self.code = code
         self.args = (code, )
-
