@@ -6,28 +6,31 @@
 # Copyright (C) 2009 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
 
-from morituri.common import logcommand, task, accurip, program, encode
+from morituri.common import logcommand, accurip, program, encode, renamer
 from morituri.image import image
 from morituri.result import result
 
+from morituri.extern.task import task
+
 
 class Encode(logcommand.LogCommand):
+
     summary = "encode image"
 
     def addOptions(self):
@@ -47,7 +50,6 @@ class Encode(logcommand.LogCommand):
                 default, "', '".join(encode.ALL_PROFILES.keys())),
             default=default)
 
-
     def do(self, args):
         prog = program.Program()
         prog.outdir = (self.options.output_directory or os.getcwd())
@@ -96,21 +98,31 @@ class Encode(logcommand.LogCommand):
                         outm3u.write('%s' % root)
                 outm3u.close()
 
+
 class Retag(logcommand.LogCommand):
+
     summary = "retag image files"
 
+    def addOptions(self):
+        self.parser.add_option('-R', '--release-id',
+            action="store", dest="release_id",
+            help="MusicBrainz release id to match to (if there are multiple)")
+
+
     def do(self, args):
-        prog = program.Program()
+        prog = program.Program(stdout=self.stdout)
         runner = task.SyncRunner()
 
         for arg in args:
-            print 'Retagging image %r' % arg
+            self.stdout.write('Retagging image %r\n' % arg)
             arg = arg.decode('utf-8')
             cueImage = image.Image(arg)
             cueImage.setup(runner)
 
             mbdiscid = cueImage.table.getMusicBrainzDiscId()
-            prog.metadata = prog.getMusicBrainz(cueImage.table, mbdiscid)
+            self.stdout.write('MusicBrainz disc id is %s\n' % mbdiscid)
+            prog.metadata = prog.getMusicBrainz(cueImage.table, mbdiscid,
+                release=self.options.release_id)
 
             if not prog.metadata:
                 print 'Not in MusicBrainz database, skipping'
@@ -120,10 +132,63 @@ class Retag(logcommand.LogCommand):
             prog.cuePath = arg
             prog.result = result.RipResult()
             for track in cueImage.table.tracks:
-                path = track.indexes[1].path
+                path = cueImage.getRealPath(track.indexes[1].path)
+
                 taglist = prog.getTagList(track.number)
-                self.debug('possibly retagging %r with taglist %r',
-                    path, taglist)
+                self.debug(
+                    'possibly retagging %r from cue path %r with taglist %r',
+                    path, arg, taglist)
+                t = encode.SafeRetagTask(path, taglist)
+                runner.run(t)
+                path = os.path.basename(path)
+                if t.changed:
+                    print 'Retagged %s' % path
+                else:
+                    print '%s already tagged correctly' % path
+            print
+
+class Rename(logcommand.LogCommand):
+
+    summary = "rename image and all files based on metadata"
+
+    def addOptions(self):
+        self.parser.add_option('-R', '--release-id',
+            action="store", dest="release_id",
+            help="MusicBrainz release id to match to (if there are multiple)")
+
+
+    def do(self, args):
+        prog = program.Program(stdout=self.stdout)
+        runner = task.SyncRunner()
+
+        for arg in args:
+            self.stdout.write('Renaming image %r\n' % arg)
+            arg = arg.decode('utf-8')
+            cueImage = image.Image(arg)
+            cueImage.setup(runner)
+
+            mbdiscid = cueImage.table.getMusicBrainzDiscId()
+
+            operator = renamer.Operator(statePath, mbdiscid)
+
+            self.stdout.write('MusicBrainz disc id is %s\n' % mbdiscid)
+            prog.metadata = prog.getMusicBrainz(cueImage.table, mbdiscid,
+                release=self.options.release_id)
+
+            if not prog.metadata:
+                print 'Not in MusicBrainz database, skipping'
+                continue
+
+            # FIXME: this feels like we're poking at internals.
+            prog.cuePath = arg
+            prog.result = result.RipResult()
+            for track in cueImage.table.tracks:
+                path = cueImage.getRealPath(track.indexes[1].path)
+
+                taglist = prog.getTagList(track.number)
+                self.debug(
+                    'possibly retagging %r from cue path %r with taglist %r',
+                    path, arg, taglist)
                 t = encode.SafeRetagTask(path, taglist)
                 runner.run(t)
                 path = os.path.basename(path)
@@ -134,6 +199,7 @@ class Retag(logcommand.LogCommand):
             print
 
 class Verify(logcommand.LogCommand):
+
     summary = "verify image"
 
     def do(self, args):
@@ -157,11 +223,13 @@ class Verify(logcommand.LogCommand):
                 tr.number = track.number
                 prog.result.tracks.append(tr)
 
-            prog.verifyImage(runner, responses) 
+            prog.verifyImage(runner, responses)
 
             print "\n".join(prog.getAccurateRipResults()) + "\n"
 
+
 class Image(logcommand.LogCommand):
+
     summary = "handle images"
 
     description = """
