@@ -6,17 +6,17 @@
 # Copyright (C) 2009 Thomas Vander Stichele
 
 # This file is part of morituri.
-# 
+#
 # morituri is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # morituri is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with morituri.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -26,9 +26,11 @@ import tempfile
 import gobject
 gobject.threads_init()
 
-from morituri.common import logcommand, task, accurip, drive, program
+from morituri.common import logcommand, accurip, drive, program
 from morituri.program import cdrdao, cdparanoia
 
+from morituri.extern.task import task
+
 # see http://www.accuraterip.com/driveoffsets.htm
 # and misc/offsets.py
 OFFSETS = "+6, +48, +102, +667, +12, +30, +618, +594, +738, -472, " + \
@@ -43,6 +45,7 @@ OFFSETS = "+6, +48, +102, +667, +12, +30, +618, +594, +738, -472, " + \
           "-495, -494, +975, +935, +87, +668, +1776, +1364, +1336, " + \
           "+1127"
 
+
 class Find(logcommand.LogCommand):
     summary = "find drive read offset"
     description = """Find drive's read offset by ripping tracks from a
@@ -77,13 +80,12 @@ CD in the AccurateRip database."""
             if not drives:
                 self.error('No CD-DA drives found!')
                 return 3
-        
+
             # pick the first
             self.options.device = drives[0]
 
         # this can be a symlink to another device
 
-
     def do(self, args):
         prog = program.Program()
         runner = task.SyncRunner()
@@ -91,7 +93,7 @@ CD in the AccurateRip database."""
         device = self.options.device
 
         # if necessary, load and unmount
-        print 'Checking device', device
+        self.stdout.write('Checking device %s\n' % device)
 
         prog.loadDevice(device)
         prog.unmountDevice(device)
@@ -104,7 +106,7 @@ CD in the AccurateRip database."""
         except cdrdao.DeviceOpenException, e:
             self.error(e.msg)
             return 3
-            
+
         table = t.table
 
         self.debug("CDDB disc id: %r", table.getCDDBDiscId())
@@ -120,7 +122,8 @@ CD in the AccurateRip database."""
             responses = accurip.getAccurateRipResponses(data)
         except urllib2.HTTPError, e:
             if e.code == 404:
-                print 'Album not found in AccurateRip database.'
+                self.stdout.write(
+                    'Album not found in AccurateRip database.\n')
                 return 1
             else:
                 raise
@@ -134,7 +137,7 @@ CD in the AccurateRip database."""
 
         # now rip the first track at various offsets, calculating AccurateRip
         # CRC, and matching it against the retrieved ones
-        
+
         def match(archecksum, track, responses):
             for i, r in enumerate(responses):
                 if archecksum == r.checksums[track - 1]:
@@ -143,13 +146,19 @@ CD in the AccurateRip database."""
             return None, None
 
         for offset in self._offsets:
-            print 'Trying read offset %d ...' % offset
+            self.stdout.write('Trying read offset %d ...\n' % offset)
             try:
                 archecksum = self._arcs(runner, table, 1, offset)
             except task.TaskException, e:
                 if isinstance(e.exception, cdparanoia.FileSizeError):
-                    print 'WARNING: cannot rip with offset %d...' % offset
+                    self.stdout.write(
+                        'WARNING: cannot rip with offset %d...\n' % offset)
                     continue
+                self.warning("Unknown exception for offset %d: %r" % (
+                    offset, e))
+                self.stdout.write(
+                    'WARNING: cannot rip with offset %d...\n' % offset)
+                continue
 
             self.debug('AR checksum calculated: %s' % archecksum)
 
@@ -157,7 +166,9 @@ CD in the AccurateRip database."""
             if c:
                 count = 1
                 self.debug('MATCHED against response %d' % i)
-                print 'Offset of device is likely %d, confirming ...' % offset
+                self.stdout.write(
+                    'Offset of device is likely %d, confirming ...\n' %
+                        offset)
 
                 # now try and rip all other tracks as well
                 for track in range(2, len(table.tracks) + 1):
@@ -165,8 +176,9 @@ CD in the AccurateRip database."""
                         archecksum = self._arcs(runner, table, track, offset)
                     except task.TaskException, e:
                         if isinstance(e.exception, cdparanoia.FileSizeError):
-                            print 'WARNING: cannot rip with offset %d...' % \
-                                offset
+                            self.stdout.write(
+                                'WARNING: cannot rip with offset %d...\n' %
+                                offset)
                             continue
 
                     c, i = match(archecksum, track, responses)
@@ -176,16 +188,16 @@ CD in the AccurateRip database."""
                         count += 1
 
                 if count == len(table.tracks):
-                    print
-                    print 'Read offset of device is: %d.' % offset
+                    self._foundOffset(device, offset)
                     return 0
                 else:
-                    print 'Only %d of %d tracks matched, continuing ...' % (
-                        count, len(table.tracks))
-                    
-        print 'No matching offset found.'
-        print 'Consider trying again with a different disc.'
-                 
+                    self.stdout.write(
+                        'Only %d of %d tracks matched, continuing ...\n' % (
+                        count, len(table.tracks)))
+
+        self.stdout.write('No matching offset found.\n')
+        self.stdout.write('Consider trying again with a different disc.\n')
+
     def _arcs(self, runner, table, track, offset):
         # rips the track with the given offset, return the arcs checksum
         self.debug('Ripping track %r with offset %d ...', track, offset)
@@ -208,13 +220,25 @@ CD in the AccurateRip database."""
         t = checksum.AccurateRipChecksumTask(path, trackNumber=track,
             trackCount=len(table.tracks))
         runner.run(t)
-        
+
         os.unlink(path)
         return "%08x" % t.checksum
 
+    def _foundOffset(self, device, offset):
+        self.stdout.write('\nRead offset of device is: %d.\n' %
+            offset)
+
+        info = drive.getDeviceInfo(device)
+        if not info:
+            return
+
+        self.stdout.write('Adding read offset to configuration file.\n')
+
+        self.getRootCommand().config.setReadOffset(info[0], info[1], info[2],
+            offset)
+
+
 class Offset(logcommand.LogCommand):
     summary = "handle drive offsets"
 
     subCommandClasses = [Find, ]
-
-
