@@ -7,8 +7,9 @@ import pickle
 import unittest
 
 from morituri.result import result
-from morituri.common import program, accurip
-from morituri.rip import cd
+from morituri.common import program, accurip, musicbrainzngs
+from morituri.rip import common as rcommon
+
 
 class TrackImageVerifyTestCase(unittest.TestCase):
     # example taken from a rip of Luke Haines Is Dead, disc 1
@@ -66,12 +67,14 @@ class TrackImageVerifyTestCase(unittest.TestCase):
             "Track 10: rip NOT accurate (max confidence      2) "
             "[16457a5a], DB [eb6e55b4]")
 
+
 class HTOATestCase(unittest.TestCase):
+
     def setUp(self):
         path = os.path.join(os.path.dirname(__file__),
             'silentalarm.result.pickle')
         self._tracks = pickle.load(open(path, 'rb'))
-        
+
     def testGetAccurateRipResults(self):
         prog = program.Program()
         prog.result = result.RipResult()
@@ -81,27 +84,30 @@ class HTOATestCase(unittest.TestCase):
 
 
 class PathTestCase(unittest.TestCase):
+
     def testStandardTemplateEmpty(self):
         prog = program.Program()
 
-        path = prog.getPath(u'/tmp', cd.DEFAULT_DISC_TEMPLATE, 'mbdiscid', 0)
+        path = prog.getPath(u'/tmp', rcommon.DEFAULT_DISC_TEMPLATE,
+            'mbdiscid', 0)
         self.assertEquals(path,
-            u'/tmp/Unknown Artist - mbdiscid/Unknown Artist - mbdiscid')
-        
+            u'/tmp/unknown/Unknown Artist - mbdiscid/Unknown Artist - mbdiscid')
+
     def testStandardTemplateFilled(self):
         prog = program.Program()
-        md = program.DiscMetadata()
+        md = musicbrainzngs.DiscMetadata()
         md.artist = md.sortName = 'Jeff Buckley'
         md.title = 'Grace'
         prog.metadata = md
 
-        path = prog.getPath(u'/tmp', cd.DEFAULT_DISC_TEMPLATE, 'mbdiscid', 0)
+        path = prog.getPath(u'/tmp', rcommon.DEFAULT_DISC_TEMPLATE,
+            'mbdiscid', 0)
         self.assertEquals(path,
-            u'/tmp/Jeff Buckley - Grace/Jeff Buckley - Grace')
+            u'/tmp/unknown/Jeff Buckley - Grace/Jeff Buckley - Grace')
 
     def testIssue66TemplateFilled(self):
         prog = program.Program()
-        md = program.DiscMetadata()
+        md = musicbrainzngs.DiscMetadata()
         md.artist = md.sortName = 'Jeff Buckley'
         md.title = 'Grace'
         prog.metadata = md
@@ -109,4 +115,3 @@ class PathTestCase(unittest.TestCase):
         path = prog.getPath(u'/tmp', u'%A/%d', 'mbdiscid', 0)
         self.assertEquals(path,
             u'/tmp/Jeff Buckley/Grace')
-  
