@@ -10,10 +10,13 @@ from morituri.image import toc
 
 from morituri.test import common
 
+
 class CureTestCase(common.TestCase):
+
     def setUp(self):
-        self.toc = toc.TocFile(os.path.join(os.path.dirname(__file__),
-            u'cure.toc'))
+        self.path = os.path.join(os.path.dirname(__file__),
+            u'cure.toc')
+        self.toc = toc.TocFile(self.path)
         self.toc.parse()
         self.assertEquals(len(self.toc.table.tracks), 13)
 
@@ -83,18 +86,32 @@ class CureTestCase(common.TestCase):
         self.toc.table.absolutize()
         cue = self.toc.table.cue()
         ref = open(os.path.join(os.path.dirname(__file__), 'cure.cue')).read()
-        self.assertEquals(cue, ref)
+        common.diffStrings(cue, ref)
 
         # we verify it because it has failed in readdisc in the past
         self.assertEquals(self.toc.table.getAccurateRipURL(),
             'http://www.accuraterip.com/accuraterip/'
             '3/c/4/dBAR-013-0019d4c3-00fe8924-b90c650d.bin')
 
+    def testGetRealPath(self):
+        self.assertRaises(KeyError, self.toc.getRealPath, u'track01.wav')
+        (fd, path) = tempfile.mkstemp(suffix=u'.morituri.test.wav')
+        self.assertEquals(self.toc.getRealPath(path), path)
+
+        winpath = path.replace('/', '\\')
+        self.assertEquals(self.toc.getRealPath(winpath), path)
+        os.close(fd)
+        os.unlink(path)
+
 # Bloc Party - Silent Alarm has a Hidden Track One Audio
+
+
 class BlocTestCase(common.TestCase):
+
     def setUp(self):
-        self.toc = toc.TocFile(os.path.join(os.path.dirname(__file__),
-            u'bloc.toc'))
+        self.path = os.path.join(os.path.dirname(__file__),
+            u'bloc.toc')
+        self.toc = toc.TocFile(self.path)
         self.toc.parse()
         self.assertEquals(len(self.toc.table.tracks), 13)
 
@@ -114,6 +131,7 @@ class BlocTestCase(common.TestCase):
         self.assertEquals(t.getIndex(1).relative, 15220)
 
     # This disc has a pre-gap, so is a good test for .CUE writing
+
     def testConvertCue(self):
         #self.toc.table.absolutize()
         self.failUnless(self.toc.table.hasTOC())
@@ -138,10 +156,14 @@ class BlocTestCase(common.TestCase):
             'e/d/2/dBAR-013-001af2de-0105994e-ad0be00d.bin')
 
 # The Breeders - Mountain Battles has CDText
+
+
 class BreedersTestCase(common.TestCase):
+
     def setUp(self):
-        self.toc = toc.TocFile(os.path.join(os.path.dirname(__file__),
-            u'breeders.toc'))
+        self.path = os.path.join(os.path.dirname(__file__),
+            u'breeders.toc')
+        self.toc = toc.TocFile(self.path)
         self.toc.parse()
         self.assertEquals(len(self.toc.table.tracks), 13)
 
@@ -164,10 +186,14 @@ class BreedersTestCase(common.TestCase):
         self.assertEquals(cue, ref)
 
 # Ladyhawke has a data track
+
+
 class LadyhawkeTestCase(common.TestCase):
+
     def setUp(self):
-        self.toc = toc.TocFile(os.path.join(os.path.dirname(__file__),
-            u'ladyhawke.toc'))
+        self.path = os.path.join(os.path.dirname(__file__),
+            u'ladyhawke.toc')
+        self.toc = toc.TocFile(self.path)
         self.toc.parse()
         self.assertEquals(len(self.toc.table.tracks), 13)
         #import code; code.interact(local=locals())
@@ -180,7 +206,32 @@ class LadyhawkeTestCase(common.TestCase):
         # c60af50d 13 150 15687 31841 51016 66616 81352 99559 116070 133243
         # 149997 161710 177832 207256 2807
 
+    def testMusicBrainz(self):
+        self.assertEquals(self.toc.table.getMusicBrainzDiscId(),
+            "KnpGsLhvH.lPrNc1PBL21lb9Bg4-")
+        self.assertEquals(self.toc.table.getMusicBrainzSubmitURL(),
+            "http://mm.musicbrainz.org/bare/cdlookup.html?toc="
+            "1+12+195856+150+15687+31841+51016+66616+81352+99559+"
+            "116070+133243+149997+161710+177832&"
+            "tracks=12&id=KnpGsLhvH.lPrNc1PBL21lb9Bg4-")
+
+    # FIXME: I don't trust this toc, but I can't find the CD anymore
+
+    def testDuration(self):
+        self.assertEquals(self.toc.table.duration(), 2761413)
+
+    def testGetFrameLength(self):
+        self.assertEquals(self.toc.table.getFrameLength(data=True), 210385)
+
+    def testCue(self):
+        self.failUnless(self.toc.table.canCue())
+        data = self.toc.table.cue()
+        lines = data.split("\n")
+        self.assertEquals(lines[0], "REM DISCID C60AF50D")
+
+
 class CapitalMergeTestCase(common.TestCase):
+
     def setUp(self):
         self.toc1 = toc.TocFile(os.path.join(os.path.dirname(__file__),
             u'capital.1.toc'))
@@ -197,19 +248,30 @@ class CapitalMergeTestCase(common.TestCase):
         self.table = copy.deepcopy(self.toc1.table)
         self.table.merge(self.toc2.table)
 
-
     def testCDDBId(self):
         self.table.absolutize()
         self.assertEquals(self.table.getCDDBDiscId(), 'b910140c')
         # output from cd-discid:
-        # b910140c 12 24320 44855 64090 77885 88095 104020 118245 129255 141765 164487 181780 209250 4440
+        # b910140c 12 24320 44855 64090 77885 88095 104020 118245 129255 141765
+        # 164487 181780 209250 4440
 
     def testMusicBrainz(self):
-        # URL to submit: http://mm.musicbrainz.org/bare/cdlookup.html?toc=1+11+197850+24320+44855+64090+77885+88095+104020+118245+129255+141765+164487+181780&tracks=11&id=MAj3xXf6QMy7G.BIFOyHyq4MySE-
+        # URL to submit: http://mm.musicbrainz.org/bare/cdlookup.html?toc=1+11+
+        # 197850+24320+44855+64090+77885+88095+104020+118245+129255+141765+
+        # 164487+181780&tracks=11&id=MAj3xXf6QMy7G.BIFOyHyq4MySE-
         self.assertEquals(self.table.getMusicBrainzDiscId(),
             "MAj3xXf6QMy7G.BIFOyHyq4MySE-")
 
+    def testDuration(self):
+        # this matches track 11 end sector - track 1 start sector on
+        # musicbrainz
+        # compare to 3rd and 4th value in URL above
+        self.assertEquals(self.table.getFrameLength(), 173530)
+        self.assertEquals(self.table.duration(), 2313733)
+
+
 class UnicodeTestCase(common.TestCase, common.UnicodeTestMixin):
+
     def setUp(self):
         # we copy the normal non-utf8 filename to a utf-8 filename
         # in this test because builds with LANG=C fail if we include
