@@ -2,11 +2,15 @@
 # vi:si:et:sw=4:sts=4:ts=4
 
 import os
-import unittest
+
+from morituri.extern.task import task
 
 from morituri.program import cdparanoia
 
-class ParseTestCase(unittest.TestCase):
+from morituri.test import common
+
+
+class ParseTestCase(common.TestCase):
 
     def setUp(self):
         # report from Afghan Whigs - Sweet Son Of A Bitch
@@ -23,7 +27,8 @@ class ParseTestCase(unittest.TestCase):
         q = '%.01f %%' % (self._parser.getTrackQuality() * 100.0, )
         self.assertEquals(q, '99.7 %')
 
-class ErrorTestCase(unittest.TestCase):
+
+class ErrorTestCase(common.TestCase):
 
     def setUp(self):
         # report from a rip with offset -1164 causing scsi errors
@@ -40,3 +45,30 @@ class ErrorTestCase(unittest.TestCase):
         q = '%.01f %%' % (self._parser.getTrackQuality() * 100.0, )
         self.assertEquals(q, '79.6 %')
 
+
+class VersionTestCase(common.TestCase):
+
+    def testGetVersion(self):
+        self.failUnless(cdparanoia.getCdParanoiaVersion())
+
+
+class AnalyzeFileTask(cdparanoia.AnalyzeTask):
+
+    def __init__(self, path):
+        self.command = ['cat', path]
+
+    def readbytesout(self, bytes):
+        self.readbyteserr(bytes)
+
+
+class CacheTestCase(common.TestCase):
+
+    def testDefeatsCache(self):
+        self.runner = task.SyncRunner(verbose=False)
+
+        path = os.path.join(os.path.dirname(__file__),
+            'cdparanoia', 'PX-L890SA.cdparanoia-A.stderr')
+        t = AnalyzeFileTask(path)
+        self.runner.run(t)
+        self.failUnless(t.defeatsCache)
+
